package Algorithms;

/**
 * Rules.java
 * Created on 15 de febrero de 2005, 21:44
 * Este objeto se encxarga de calcular el valor de las relaciones en el caso de los mapas dinamicos
 * @author Joso Bladimir Contreras Vargas
 * @author <A HREF="mailto:jbc_ve@hotmail.com">jbc_ve@hotmail.com</A>
 */
public class Rules {
    /**
     *Funcion que determina si un el valor de v es mayor a 2/3
     *@param v valor a determinar si se encuentra en el rango
     *@return cierto si el valor de v es mayor a 2/3 falso en caso contrario
     */
    static private boolean isHigh ( float v ){
        return ( v > 2.0f / 3.0f);
    }
    
    /**
     *Funcion que determina si un el valor de v es menor a 1/3
     *@param v valor a determinar si se encuentra en el rango
     *@return cierto si el valor de v es menor a 1/3 falso en caso contrario
     */
    static private boolean isLow ( float v ){
        return ( v <= 1.0f / 3.0f);
    }
    
    /**
     *Funcion que determina si un el valor de v se encuentra en el rango (1/3 2/3]
     *@param v valor a determinar si se encuentra en el rango
     *@return cierto si el valor de v se encuentra en el rango (1/3 2/3]
     */
    static private boolean isMedium ( float v ){
        return ( v > 1.0f/3.0f && v <= 2.0f/3.0f );
    }
    
    /**
     * Funcion que determina el valor de la relacion, de acuerdo a las reglas especificadas
     * @param relation relacion a la cual se lengthva a calcular su valor
     * @return el valor de la relacion
     */
    static public float computeRelationValue ( Map.Relation  relation ){
        
        if ( relation.getInitialConcept().getName().equals("b/A") && relation.getFinalConcept().getName().equals("H"))
            relation.setValor ( relation.getDinamicImput() );
        else if ( relation.getInitialConcept().getName().equals("a/A") && relation.getFinalConcept().getName().equals("H"))
            relation.setValor (  -1.0f*(float)Math.sqrt( relation.getFinalConcept().getCurrentValue() )  );
        else
            relation.setValor ( 1.0f );
        
        
        /*
        float vInicial = relation.getInitialConcept().getCurrentValue();
        float vFinal = relation.getFinalConcept().getCurrentValue();
        
        if ( isHigh ( vInicial ) && isHigh( vFinal ) )
            relation.setValor( 1.0f );
        else if ( isHigh ( vInicial ) && isMedium ( vFinal ) )
            relation.setValor( 0.75f );
        else if ( isHigh ( vInicial ) && isLow( vFinal ) )
            relation.setValor( 0.25f );
        
        else if ( isMedium ( vInicial )&& isHigh( vFinal ) )
            relation.setValor( 0.75f );
        else if ( isMedium ( vInicial ) && isMedium ( vFinal ) )
            relation.setValor( -0.5f );
        else if ( isMedium ( vInicial ) && isLow( vFinal ) )
            relation.setValor( -0.75f );
        
        else if ( isLow( vInicial ) && isHigh( vFinal ) )
            relation.setValor( -0.75f );
        else if ( isLow( vInicial ) && isMedium ( vFinal ) )
            relation.setValor( -0.5f );
        else if ( isLow( vInicial ) && isLow( vFinal ) )
            relation.setValor( -1.0f );
        */
        relation.setValor ( relation.getValue() * relation.getDinamicImput()  );
        return relation.getValue();
         
    }
}
