package Interface.MultiCapas;

import java.awt.Label;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;

import javax.swing.JDialog;
import javax.swing.JFrame;

import Algorithms.RunFCM;
import Interface.MapCanvas;
import Map.Concept;
import Map.Map;
import PanelInterfaceMulticapa.PanelReglas;
import PanelInterfaceMulticapa.PanelResultados;

public class InterfazMultiCapas {

	private ArrayList<ReglasConexion> listaReglasConexion;
	private ArrayList<ReglasEcuacion> listaReglasEcuaciones;
	private ArrayList<ReglasDifusas> listaReglasDifusas;
	private RunFCM rumMapa1;
	private RunFCM rumMapa2;
	private int id;
	private int maximoIteraciones = 10;
	private ArrayList<Float> listaActual;
	private ArrayList<ArrayList<Float>> listaFinal;
	private JFrame principal;
	private MapCanvas mapa1;
	private MapCanvas mapa2;

	public InterfazMultiCapas(JFrame principal) {
		this.principal = principal;
		listaReglasConexion = new ArrayList<ReglasConexion>();
	}

	private void inicializar() {
		listaActual = new ArrayList<Float>();
		listaFinal = new ArrayList<ArrayList<Float>>();
	}

	public void AsignarDatos(MapCanvas mapa1, String iteracionMapa1,
			MapCanvas mapa2, String iteracionMapa2) {
		mapa1.getMap().setIdMapa(1);
		mapa2.getMap().setIdMapa(2);
		this.mapa1 = mapa1;
		this.mapa2 = mapa2;
		rumMapa1 = new RunFCM(mapa1, new Label(iteracionMapa1), this);
		rumMapa2 = new RunFCM(mapa2, new Label(iteracionMapa2), this);
	}

	private Boolean condition(int iterardor) {
		if (VerificarConvergencia())
			return false;
		if (iterardor > maximoIteraciones)
			return false;
		return true;
	}

	@SuppressWarnings("unchecked")
	private Boolean VerificarConvergencia() {
		int pos = 0;
		ArrayList<Concept> listconcepts = rumMapa1.getMapa().getConceptList();
		for (Concept concept : listconcepts) {
			if (concept.getLastValue() != listaActual.get(pos)) {
				return false;
			}
			pos++;
		}
		listconcepts = rumMapa2.getMapa().getConceptList();
		for (Concept concept : listconcepts) {
			if (concept.getLastValue() != listaActual.get(pos)) {
				return false;
			}
			pos++;
		}
		return true;
	}

	//
	@SuppressWarnings("unchecked")
	private void ActualizarListaActual() {
		if (listaActual.size() > 0)
			listaFinal.add((ArrayList<Float>) listaActual.clone());
		listaActual.clear();
		ArrayList<Concept> listconcepts = rumMapa1.getMapa().getConceptList();
		for (Concept concept : listconcepts) {
			listaActual.add(concept.getLastValue());
		}
		listconcepts = rumMapa2.getMapa().getConceptList();
		for (Concept concept : listconcepts) {
			listaActual.add(concept.getLastValue());
		}
	}

	public void Run() {
		inicializar();
		int iterardorAux = 0;
		do {
			ActualizarListaActual();
			ejecutarComandos();
			rumMapa1.run();
			rumMapa2.run();
			iterardorAux++;
		} while (condition(iterardorAux));
		ActualizarListaActual();
		VentanaResultados();

	}

	public void asignarNormalizacion(int NormalizacionMapa1,
			int NormalizacionMapa2) {
		rumMapa1.tipoSaturacion = NormalizacionMapa1;
		rumMapa2.tipoSaturacion = NormalizacionMapa2;
	}

	private void ejecutarComandos() {
		if (id == 1) {
			ejecutarComandosEcuacion();
			return;
		}
		if (id == 2) {
			ejecutarComandosDifuso();
		}
	}

	private void ejecutarComandosEcuacion() {
		for (ReglasEcuacion rE : listaReglasEcuaciones) {
			rE.calcularPeso();
		}
	}

	private void ejecutarComandosDifuso() {
		for (ReglasDifusas rD : listaReglasDifusas) {
			rD.calcularPeso();
		}
	}

	public Float getValuInterfaz(Concept concept, int idMapaDestino) {
		Float dato = (float) 0;
		switch (id) {
		case 0:
			for (ReglasConexion rC : listaReglasConexion) {
				if (concept.getConceptID() == rC.getConceptDestino()
						.getConceptID()
						&& idMapaDestino == rC.getIdMapaDestino()) {
					dato = dato + rC.valorRegla();
				}
			}

			break;

		case 1:
			for (ReglasEcuacion rE : listaReglasEcuaciones) {
				if (concept.getConceptID() == rE.getDestino().getConceptID()
						&& idMapaDestino == rE.getIdMapaDetino())
					dato = dato + rE.getPeso();
			}

			break;

		case 2:
			for (ReglasDifusas rD : listaReglasDifusas) {
				if (concept.getConceptID() == rD.getDestino().getConceptID()
						&& idMapaDestino == rD.getIdMapaDetino())
					dato = dato + rD.getPeso();
			}

			break;

		default:
			System.err.println("Algo estubo mal");
		}

		return dato;
	}

	private void VentanaResultados() {
		JDialog ventanaRes = new JDialog(principal, "Resultados de Simulacion",
				true);
		ventanaRes.setSize(700, 600);
		ventanaRes.setLocationRelativeTo(null);
		ventanaRes.add(new PanelResultados(this));
		ventanaRes.setVisible(true);
		ventanaRes.addWindowListener(new WindowAdapter() {
			public void windowDeactivated(WindowEvent e) {
				reiniciarElementos();
			}
		});
	}

	private void reiniciarElementos() {
		rumMapa1.getMapa().resetConcepts();
		rumMapa2.getMapa().resetConcepts();
		listaActual = new ArrayList<Float>();
	}

	public ArrayList<ReglasConexion> getListaReglasConexion() {
		return listaReglasConexion;
	}

	public void setListaReglasConexion(
			ArrayList<ReglasConexion> listaReglasConexion) {
		id = 0;
		this.listaReglasConexion = listaReglasConexion;
	}

	public ArrayList<ReglasEcuacion> getListaReglasEcuaciones() {
		return listaReglasEcuaciones;
	}

	public void setListaReglasEcuaciones(
			ArrayList<ReglasEcuacion> listaReglasEcuaciones) {
		id = 1;
		this.listaReglasEcuaciones = listaReglasEcuaciones;
	}

	public ArrayList<ReglasDifusas> getListaReglasLogica() {
		return listaReglasDifusas;
	}

	public void setListaReglasDifusas(
			ArrayList<ReglasDifusas> listaReglasDifusas) {
		id = 2;
		this.listaReglasDifusas = listaReglasDifusas;
	}

	public Map getMap(int idMapa) {
		if (idMapa == 1) {
			return rumMapa1.getMapa();
		}
		return rumMapa2.getMapa();
	}

	public ArrayList<ArrayList<Float>> getListaFinal() {
		return listaFinal;
	}

	public void setMaximoIteraciones(int iteraciones) {
		this.maximoIteraciones = iteraciones;
	}

	public JFrame getPrincipal() {
		return principal;
	}

	public MapCanvas getMapaCanvas(int idMapa) {
		if (idMapa == 1) {
			return mapa1;
		}
		return mapa2;
	}

}
