package Interface.MultiCapas;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.InputStreamReader;
import java.util.ArrayList;

import javax.swing.JOptionPane;
import javax.swing.JPanel;

import Map.Concept;

public class ReglasEcuacion {

	private Float peso;
	private ArrayList<MapeoConceptos> paramteros;
	private Concept destino;
	private int idMapaDetino;
	private String formula;
	private Boolean error = false;
	private String mensajeError;

	public ReglasEcuacion(ArrayList<MapeoConceptos> paramteros,
			Concept destino, int idMapaDetino, String formula) {
		this.paramteros = paramteros;
		this.setDestino(destino);
		this.setIdMapaDetino(idMapaDetino);
		this.formula = formula;
	}

	public void calcularPeso() {
		actualizarArcivo();
		ejecutarConsola();
	}

	private void actualizarArcivo() {
		try {
			File archivo = new File("ecuaciones.m");
			BufferedWriter escribir = new BufferedWriter(
					new FileWriter(archivo));

			for (MapeoConceptos mC : paramteros) {
				escribir.write(mC.getId() + "="
						+ mC.getConcept().getCurrentValue() + ";\r\n");
			}
			escribir.write("f=" + formula + "\r\n");
			escribir.close();
		} catch (Exception e) {
			System.out.println("Error al escribir");
		}
	}

	private void ejecutarConsola() {
		String s = null;
		ArrayList<String> listElem = new ArrayList<String>();
		try {
			Runtime command = Runtime.getRuntime();
			Process pr = command.exec("octave -q ecuaciones.m");
			BufferedReader stdInput = new BufferedReader(new InputStreamReader(
					pr.getInputStream()));
			BufferedReader stdError = new BufferedReader(new InputStreamReader(
					pr.getErrorStream()));
			while ((s = stdInput.readLine()) != null) {
				listElem.add(s);
			}
			if (pr.waitFor() != 0) {
				mensajeError = stdError.readLine();
				while ((s = stdError.readLine()) != null) {
					System.out.println(s);
				}
				error = true;
			} else {
				String[] e = listElem.get(listElem.size() - 1).split("=");
				peso = Float.parseFloat(e[e.length - 1]);
			}

		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public Float getPeso() {
		return peso;
	}

	public void setPeso(Float peso) {
		this.peso = peso;
	}

	public Concept getDestino() {
		return destino;
	}

	public void setDestino(Concept destino) {
		this.destino = destino;
	}

	public int getIdMapaDetino() {
		return idMapaDetino;
	}

	public void setIdMapaDetino(int idMapaDetino) {
		this.idMapaDetino = idMapaDetino;
	}

	public Boolean getError() {
		return error;
	}

	public void mostrarError(JPanel panel) {
		JOptionPane jop = new JOptionPane();
		jop.showMessageDialog(
				panel,
				"Error: "+mensajeError
						+ "\r\nPosible solucion:\r\nVerificar sintaxis de la formula."
						+ "\r\nVerificar si todas las variables estan representadas por conceptos."
						+ "\r\nVerificar que las variables no inicien con digitos."
						+ "\r\nVerificar si todas las lines del panel formula terminan en \";\"",
				"Error en Construccion de Reglas de Ecuacuaciones",
				JOptionPane.ERROR_MESSAGE);
	}

}
