package Map;

/**
 *Esta Clase Nodo es utilizada para representar un nodo en un grafo dirigido
 *dicho grafo sera utilizado como un Mapa Cognitivo Difuso el cual itera en el tiempo
 *por ello el este objeto presenta una lista de valores la cual contiene
 *los valores en el tiempo de vida del nodo
 *Created on 25 de enero de 2005, 11:36
 * @author Jos Bladimir Contreras Vargas
 * @author <A HREF="mailto:jbc_ve@hotmail.com">jbc_ve@hotmail.com</A>
 */
public class Concept{
    
    /**
     * Variable para generar los identificadores
     */
    private static int nodeID = 0;          
    /**
     * identificador del nodo
     */
    private int identificador;              
    /**
     * posicin izquierda superior del nodo
     */
    private java.awt.Point position;        
    /**
     * Tamalo Maximo
     */
    private static final int MAXSIZE = 20;  
    /**
     * Constante para indicar que la posicin de la etiqueta sea Arriba del Concepto
     */
    public static final int ARRIBA = 0 ;    
    /**
     * Constante para indicar que la posicin de la etiqueta sea a un lado Concepto
     */
    public static final int RSIDE = 1 ;    
    public static final int LSIDE = 3 ;
    /**
     * Constante para indicar que la posicin de la etiqueta sea Abajo del Concepto
     */
    public static final int ABAJO = 2 ;     
    /**
     * variable para indicar la posicin de la etiqueta
     */
    private int posicionEtiqueta = this.LSIDE;  
    /**
     * radio del nodo
     */
    private int radio = 10;                      
    /**
     * nombre del nodo
     */
    private String nombre;                  
    /**
     * valor actual del nodo
     */
    private float valorActual = 1;          
    /**
     * valor inical del nodo
     */
    private float valorInicial = this.valorActual;
    /**
     * valor auxiliar dek nodo
     */
    public float valorAuxiliar = 0;
    /**
     * lista de valores durante la ejecucin
     */
    private java.util.LinkedList valores;   //Lista de valores
    /**
     * Comentario del nodo
     */
    public String comentario = "";
    /**
     * indica si el nodo pertenece a un mapa dinamico o no
     */
    private boolean dynamic = false;
    
    /**
     * Crea un Nodo por defecto en la posicion indicada, a dicho nodo se le
     * asigna un identificador unico el cual es enlazado al nombre por defecto
     * tambien se lengthasigna un radio por defecto de 10 px y un valor Inicial de 1
     * @param x Posicion x
     * @param y Posicion y
     * @param dynamic true si el concepto pertenece al un mapa dinamico
     * false en caso contrario
     */
    public Concept (int x , int y , boolean dynamic ){
        this.position = new java.awt.Point ( );
        this.setPosition( x , y );
        this.identificador = this.nodeID++;
        this.nombre = "Node " + this.identificador;   
        this.valores = new java.util.LinkedList();
        this.valores.add( new Float (this.valorInicial) );
        this.dynamic = dynamic;
    }
    
    /**
     * Asignar el nombre al Concepto
     * @param _nombre Nombre a asignar
     */
    public void setName ( String _nombre ){
        this.nombre = _nombre;
    }
    
    /**
     * Obtener el nombre del nodo
     * @return String que contiene el nombre del Concepto
     */
    public String getName (){
        return this.nombre;
    }
    
    /**
     * Retorna el ultimo valor en la lista de valores
     * @return valor flotante con el ultimo va
     */
    public float getLastValue (){
        return  ( (Float)this.valores.getLast() ).floatValue();
    }
    
    /**
     *Agrega value de ultimo a la lista de valores, tambien lo asigana como valor Actual
     *@param value Valor a agregar como ultimo valor y como valor actual
     */
    public void addLastValue ( float value ){
        this.valores.addLast( new Float ( value ) );
        this.valorActual = value;
    }
    
    /**
     *Asignar la posicion de la etiqueta
     *@param p posicion de la etiqueta utilizar las constantes Node.ARRIBA, Node.ABAJO, Node.LADO
     */
    public void setLabelPosition ( int p ){
        if ( p == this.ABAJO )
            this.posicionEtiqueta = this.ABAJO;
        else if ( p == this.RSIDE )
            this.posicionEtiqueta = this.RSIDE;
        else if ( p == this.LSIDE )
            this.posicionEtiqueta = this.LSIDE;
        else 
            this.posicionEtiqueta = this.ARRIBA;
    }
    
    /**
     * Retorna en que posicion se encuentra la etiqueta
     * se deben usat las constantes Node.ARRIBA, Node.ABAJO, Node.LADO
     * @return Retorna la posicion de la etiqueta del concepto
     */
    public int getLabelPosition (){
        return this.posicionEtiqueta;
    }
    
    /**
     * Obtener la posicion del punto superior izquierdo del concepto en el lienzo
     * @return Punto con la posicin del nodo
     */
    public java.awt.Point getPosition (){
        return this.position;
    }
    
    /**
     * Asignar la posicion del nodo en el lienzo
     * @param x posicion x
     * @param y posicion y
     */
    public void setPosition ( int x , int y ){
        this.position.setLocation( x , y );
        if ( this.position.x < 0 )
            this.position.x = 0;
        else if ( this.position.x > 1024-2*this.radio )
            this.position.x = 1024-2*this.radio;
        
        if ( this.position.y < 0 )
            this.position.y = 0;
        else if ( this.position.y > 1024-2*this.radio )
            this.position.y = 1024-2*this.radio;
    }
       
    /**
     *Asigna un radio al nodo
     *@param r radio a asignar r>=5 r<=Nodo.MAXSIZE+5 
     */
    public void setRadio ( int r ){
        if ( !(r < 5) && !(r>this.MAXSIZE+5) )
            this.radio = r;
    }
    
    /**
     * Obtener el radio del nodo
     * @return retorna el radio actual del concepto
     */
    public int getRadio () {
        return this.radio;
    }
    
    /**
     * Asigna un Identificador al nodo
     * OJO esta funcion es solo para uso interno de la clase graph
     * @param i indica el identificador del concepto
     */
    public void setID ( int i ){
        this.identificador = i;
    }
    
    /**
     * Obtener el identificador del nodo
     * @return Obtiene el identificador del Concepto
     */
    public int getConceptID(){
        return this.identificador;
    }
      
    /**
     *Asignar el Valor Inicial al nodo, la lista de valores se reinicia
     *@param value Valor a asignar como valor inicial
     */
    public void setInitialValue ( float value ){
        if ( value >= 0.0f && value <= 1.0f)
            this.valorInicial = value;
        else if ( value > 1.0f )
            this.valorInicial = 1.0f;
        else
            this.valorInicial = 0.0f;
        this.valores.clear();
        this.valores.addFirst( new Float ( this.valorInicial ) );
    }
    
    /**
     * Obtener el valor Inicial del nodo
     * @return valor inicial del concepto
     */
    public float getInitialValue (){
        return this.valorInicial;
    }
     
    /**
     * Asignar valor actual, no modifica la lista de valores solo la variable valor Actual
     * Usada por RunFCM
     * @param value valor a asignar
     */
    public void setCurrentValue ( float value ){
     
         if ( value >= 0.0f && value <= 1.0f)
            this.valorActual = value;
        else if ( value > 1.0f )
            this.valorActual = 1.0f;
        else
            this.valorActual = 0.0f;
//        System.out.println("id de mapa :" + identificador + " tamao actual: " + valores.size());
    }
    
    /**
     * Obtener el valor actual del nodo
     * @return Valor actual del nodo
     */
    public float getCurrentValue(){
//      System.out.println(valores.size());
        return this.valorActual;
    } 
    
    /**
     * Indica si la posicion se encuentra dentro de los limites del nodo
     * @param x posicion x
     * @param y posicion y
     * @return retorna si esa posicion se encuentra dentro del rectangulo que contiene el concepto
     */
    public boolean dentro ( int x , int y ){
        int tam = 2 * this.radio;
        return (( x >= this.position.x && x <= this.position.x + tam ) && 
                ( y >= this.position.y && y <= this.position.y + tam ) ); 
    }
    
    /**
     *Agrega el Nodo y su etiqueta en el objeto g, la variable this.radio es modificada
     *@param g Objeto grafico donde se va a dibujar el nodo
     *@param conValor true el radio se calcula de acuerdo al valor Actual del nodo false el radio por defecto 10px
     */
    public void drawConcept ( java.awt.Graphics g , boolean conValor ){
        
        int tam;
        if ( conValor )
            tam = Math.round( (5 + this.MAXSIZE * this.valorActual )*2 );
        else
            tam = 20;
        
        this.radio = tam/2;
        
        g.drawOval( this.position.x , this.position.y , tam , tam );
        int stringtam = this.nombre.length() *6;
        if ( this.posicionEtiqueta == this.ABAJO )
            g.drawString( this.nombre , this.position.x + radio - stringtam/2 , this.position.y + tam + 12 );
        else if ( this.posicionEtiqueta == this.ARRIBA )
            g.drawString( this.nombre , this.position.x + radio - stringtam/2  , this.position.y - 2 );
        else if ( this.posicionEtiqueta == this.RSIDE )
            g.drawString( this.nombre , this.position.x + tam + 2 , this.position.y + tam/2 + 6 );
        else
            g.drawString( this.nombre , this.position.x - stringtam - 12 , this.position.y + tam/2 + 6 );
    }
    
    /**
     * Reinicia la lista de valores de la ejecucin del nodo
     */
    public void reset(){
        this.valores.clear();
        this.valores.add( new Float (this.valorInicial));
        this.valorActual = this.valorInicial;
    }
    
    /**
     * retorna cuantos valores hay en la lista de ejecucin
     * @return Cantidad de valores en la lista de ejecucin
     */
    public int getValoresSize(){
        return this.valores.size();
    }
    
    /**
     * Coloca como valor actual el valor de la iteracin indicada
     * @param i iteracin a colocar como actual
     */
    public void setCurrentIteration ( int i ){
        if ( i >= this.valores.size() || i < 0 )
            return;
        else
            this.valorActual = ((Float)this.valores.get(i)).floatValue();
    }
    
    /**
     * retorna el valor del concepto en la iteracin indicada
     * @return valor del concepto en la iteracion
     * @param i iteracin
     */
    public float getValue ( int i ){
        return ( ((Float)this.valores.get( i )).floatValue());
    }
    
    /**
     * Asigna un comentario al concepto
     * @param c String con el comentario que identifica al concepto no se puede usar el caracter &
     */
    public void setComentario ( String c ){
        this.comentario = c.replace( '\n','&');
        this.comentario = this.comentario.replace((char)13 , ' ');
    }
    
    /**
     * retorna el comentario del concepto
     * @return String que contiene el comentario del concepto
     */
    public String getComentario (){
        return this.comentario.replace( '&' , '\n');
    }
    
}
