package Map;

/**
 * Objeto Utilizado para Represenatr el mapa cognitivo difuso
 * Created on 30 de enero de 2005, 23:01
 * @author Jos Bladimir Contreras Vargas
 * @author <A HREF="mailto:jbc_ve@hotmail.com">jbc_ve@hotmail.com</A>
 */
public class Map {
    /**
     * iteracion actualmente mostrada
     */
    private int currentIteration=0;
    /**
     * lista de conceptos que compone el mapa
     */
    private java.util.ArrayList conceptsList = new java.util.ArrayList();
    /**
     * lista de relaciones que compone el mapa
     */
    private java.util.ArrayList relationsList = new java.util.ArrayList();
    /**
     * indica si el mapa es dinamico o no
     */
    private boolean dynamic = false;
    
    private int idMapa;
    
    /**
     * crea un mapa vacio
     * @param dynamic indica si el mapa debe ser dinamico o no
     */
    public Map(boolean dynamic) { 
        this.dynamic = dynamic;
    }
    
    /**
     * retorna si el mapa es dinamico o no
     * @return true - dinamico
     * false - no dinamico
     */
    public boolean isDynamic(){
        return this.dynamic;
    }
    
    /**
     * agrega un concepto al nodo
     * @param x posicion x del concepto
     * @param y posicion y del concepto
     * @param dynamic indica si el mapa es dinamico o no
     */
    public void addConcept ( int x , int y , boolean dynamic){
        this.conceptsList.add ( new Concept ( x , y , dynamic) );
    }
    
    /**
     * agrega una relacin al mapa
     * @param nodoInicial Concepto Antecedente
     * @param nodoFinal Concepto Consecuente
     */
    public void addRelation ( Concept nodoInicial , Concept nodoFinal ){
        this.relationsList.add( new Relation ( nodoInicial, nodoFinal , this.dynamic) );
    }
    
    /**
     * indica si existe algun concepto en la posicion indicada
     * @param x posicion x
     * @param y posicion y
     * @return Concepto encontrado null en caso de que no se halla encontrado
     */
    public Concept findConcept ( int x, int y){
        //Node ref; 
        java.util.Iterator <Concept> it = this.conceptsList.iterator();
        while (  it.hasNext()  ){
             Concept ref = it.next();
             if ( ref.dentro( x , y ) )
                 return ref;
        }
        return null;
    }
    
    /**
     * Busca un Concepto por su identificador
     * @param id identificador a encontrar
     * @return Concepto encontrado null en caso de que no se halla encontrado
     */
    public Concept findConcept ( int id){
        java.util.Iterator <Concept> it = this.conceptsList.iterator();
        while (  it.hasNext()  ){
             Concept ref = it.next();
             if ( ref.getConceptID() == id )
                 return ref;
        }
        return null;
    }
    
    /**
     * Busca el eje que se encuentra en la posicion indicada
     * @param x posicion x
     * @param y posicion y
     * @return Relacion buscada o null
     */
    public Relation findRelation ( int x , int y ){
        java.util.Iterator <Relation> it = this.relationsList.iterator();
        while (  it.hasNext()  ){
             Relation ref = it.next();
             if ( ref.dentro( x , y ) )
                 return ref;
        }
        return null;
    }
    
    /**
     * elimina la relacin
     * @param ID relacin a eliminar
     */
    public void deleteRelation ( Relation ID ){
        for ( int i = 0 ; i < this.relationsList.size() ; i++ )
            if ( ((Relation)this.relationsList.get( i )) == ID ){
                this.relationsList.remove( i );
                break;
            }
    }
    
    /**
     * elimina un Concepto
     * @param ID Concepto a eliminar
     */
    public void deleteConcept ( Concept ID ){
        int i;
        for ( i = 0 ; i < this.conceptsList.size() ; i++ )
            if ( ((Concept)this.conceptsList.get( i )) == ID ){
                this.conceptsList.remove( i );
                break;
            }
        for ( i = 0 ; i < this.relationsList.size() ; i++ ){
            if ( i < 0 ) i = 0;
            if ( ((Relation)this.relationsList.get(i)).getFinalConcept()  == ID || 
                 ((Relation)this.relationsList.get(i)).getInitialConcept() == ID ){
                this.relationsList.remove( i );
                i--;
            }
        }
    
        
    }
          
    /**
     * Dibuja el mapa
     * @param g elemento grafico donde se dibujara el mapa
     * @param conValor indica si los conceptos se deben dibujar proporcionalmente a su valor
     */
    public void drawMap ( java.awt.Graphics g , boolean conValor ){
        java.util.Iterator <Concept> it = this.conceptsList.iterator();
        while ( it.hasNext() ){
            it.next().drawConcept( g , conValor );
        }
        
        java.util.Iterator <Relation> i = this.relationsList.iterator();
        while ( i.hasNext() ){
            i.next().drawRelation( g );
        }
    }
    
    /**
     * elimina todo el mapa
     */
    public void clear (){
        this.relationsList.clear();
        this.conceptsList.clear();
        this.currentIteration = 0;
    }
    
    /**
     * actualiza los indentificadores de los nodos
     */
    public void updateConceptsID(){
        java.util.Iterator <Concept> it = this.conceptsList.iterator();
        int i = 0;
        while (  it.hasNext()  ){
             Concept ref = it.next();
             ref.setID( i++ );
        }
    }
    
    /**
     * Guarda el mapa en un archivo
     * @param dir Directorio del archivo
     * @param file Nombre del archivo
     */
    public void save ( String dir , String file ){
        this.updateConceptsID();
        java.io.FileWriter out = null;
        try{
            out = new java.io.FileWriter( dir+"/"+file);
            String graphInf = ""+this.conceptsList.size() + "\n" + this.relationsList.size() + "\n";
            out.write(graphInf);
            String nodoInf;
            java.util.Iterator <Concept> it = this.conceptsList.iterator();
            while (  it.hasNext()  ){
                 Concept ref = it.next();
                 nodoInf = "" + 
                       ref.getName() + "\n" +
                       ref.getConceptID() + "\n" +
                       ref.getPosition().x + "\n" +
                       ref.getPosition().y  + "\n" +
                       ref.getRadio() + "\n" +
                       ref.getInitialValue() + "\n" +
                       ref.getLabelPosition() + "\n" +
                       ref.comentario + "\n";
                 out.write( nodoInf );
            }
        
            String edgeInf;
            java.util.Iterator <Relation> ite = this.relationsList.iterator();
            while (  ite.hasNext()  ){
                 Relation ref = ite.next();
                 edgeInf = "" + 
                        ref.getInitialConcept().getConceptID() + "\n" +
                        ref.getFinalConcept().getConceptID() + "\n" +
                        ref.getValue() + "\n" +
                        ref.getPorcentajeAproximacion() + "\n" +
                        ref.getDinamicImput() + "\n" + 
                        ref.comentario + "\n";
                 out.write( edgeInf);           
            }   
        }catch (java.io.IOException ex) {
            System.out.println(ex);
       }finally{
           if ( out != null )
               try{
                   out.close();
               }catch (java.io.IOException ex) {
                   System.out.println(ex);
               }
       }
    } 
     
    /**
     * Carga el mapa en un archivo
     * @param dir Directorio del archivo
     * @param file Nombre del archivo
     */
    public void load ( String dir , String file ){
        this.currentIteration = 0 ;
        this.clear();
        int nodes , edges;
        try{
               java.io.File f = new java.io.File (dir+"/"+file);
               java.io.BufferedReader in = new java.io.BufferedReader (  new java.io.FileReader (f) );
               String s = in.readLine();
               nodes = Integer.valueOf(s).intValue();
               s = in.readLine();
               edges = Integer.valueOf(s).intValue();
               for ( int i = 0 ; i < nodes && s != null ; i++ ){
                   Concept n = new Concept(0,0, this.dynamic );
                   s = in.readLine();
                   n.setName( s );
                   s = in.readLine();
                   n.setID( Integer.valueOf(s).intValue() );
                   java.awt.Point p = new java.awt.Point();
                   s = in.readLine();
                   p.x = Integer.valueOf(s).intValue();
                   s = in.readLine();
                   p.y = Integer.valueOf(s).intValue();
                   n.setPosition(p.x,p.y);
                   s = in.readLine();
                   n.setRadio(Integer.valueOf(s).intValue());
                   s = in.readLine();
                   n.setInitialValue( Float.valueOf(s) );
                   n.setCurrentValue( Float.valueOf(s) );
                   s = in.readLine();
                   n.setLabelPosition( Integer.valueOf(s).intValue() );
                   s = in.readLine();
                   n.setComentario(s);
                   this.conceptsList.add( n );
               }
               
               Concept nodoInicial, nodoFinal;
               String nI, nF;
               for ( int i = 0 ; i < edges && s != null ; i++ ){
                   nI = in.readLine();
                   nF = in.readLine();
                   nodoInicial = this.findConcept ( Integer.valueOf(nI).intValue() );
                   nodoFinal = this.findConcept ( Integer.valueOf(nF).intValue() );
                   Relation e = new Relation(nodoInicial , nodoFinal , this.dynamic );
                   s = in.readLine();
                   e.setValor( Float.valueOf(s) );
                   s = in.readLine();
                   e.setPorcentajeAproximacion( Float.valueOf(s) );
                   s = in.readLine();
                   e.setDinamicImput( Float.valueOf(s) );
                   s = in.readLine();
                   e.setComentario(s);
                   this.relationsList.add(e);
               }
               in.close();
           } catch ( java.io.IOException ee){
               System.out.println( ee );
           }
    } 
    
    /**
     * Retorna la lista de Relaciones
     * @return Lista de Relaciones
     */
    public java.util.ArrayList getRelationList (){
        return this.relationsList;
    }
    
    /**
     * Retorna la lista de Conceptos
     * @return Lista de Conceptos
     */
    public java.util.ArrayList getConceptList (){
        return this.conceptsList;
    }
    
    /**
     * Lismpia la lista de Conceptos
     */
    public void resetConcepts (){
        java.util.Iterator <Concept> it = this.conceptsList.iterator();
        while (  it.hasNext()  ){
             Concept ref = it.next();
             ref.reset();
        }
        this.setShowIteration(0);
    }
    
    /**
     * Retorna el tamo de la lista de ejecucin de los conceptos
     * @return cantidad de iteraciones
     */
    public int sizeConceptsData (){
        if ( this.conceptsList.isEmpty() )
            return 1;
        
        return ((Concept)this.conceptsList.get(0)).getValoresSize();
    }
    
    /**
     * Asigna como iteracion a mostar
     * @param i numero de la iteracion a mostrar
     */
    public void setShowIteration( int i ){
        if ( i >= this.sizeConceptsData() || i < 0)
            return;
        java.util.Iterator <Concept> it = this.conceptsList.iterator();
        while (  it.hasNext()  )
             it.next().setCurrentIteration(i);
        this.currentIteration = i;
    }
    
    /**
     * retorna el numero de la iteracion actual
     * @return iteracion actual
     */
    public int getCurrentIteration (){
        return this.currentIteration;
    }
    
    /**
     * <B>Solo para uso de RunFCM</B>
     * @param i numero de la iteracion
     */
    public void setCurrentIteration (int i){
        this.currentIteration = i;
    }
    
    /**
     * retorna la etiqueta de las iteraciones
     * @return String con la iteracion actual
     */
    public String getIterationLabel (){
        return (""+this.currentIteration+" / " + (this.sizeConceptsData()-1) );
    }
    
    /**
     * Guarda la ejecucin del mapa
     * @param dir Directorio
     * @param file nombre del archivo
     */
    public void saveExecution ( String dir , String file ){
        this.updateConceptsID();
        java.io.FileWriter out = null;
        try{
            out = new java.io.FileWriter( dir+"/"+file);
            
            float Matriz[][] = new float [this.conceptsList.size()][this.conceptsList.size()];
            
            java.util.Iterator <Relation> ite = this.relationsList.iterator();
            while (  ite.hasNext()  ){
                 Relation ref = ite.next();
                 Matriz[ ref.getInitialConcept().getConceptID()][ref.getFinalConcept().getConceptID()] = ref.getValue();
            }
            
            for ( int i = 0 ; i < this.conceptsList.size() ; i++ ){
                for ( int j = 0 ; j < this.conceptsList.size() ; j++ )
                    out.write(Matriz[i][j]+" ");
                out.write ("\n");
            }
            
            java.util.Iterator <Concept> itn = this.conceptsList.iterator();
            while ( itn.hasNext() )
                out.write ( itn.next().getName() + "\n" );
            out.write("C = [ ");
            for ( int i = 0 ; i < this.sizeConceptsData() ; i++ ){
                java.util.Iterator <Concept> itn1 = this.conceptsList.iterator();
                //out.write("C"+i+" = [ ");
                while ( itn1.hasNext() )
                    out.write( itn1.next().getValue(i) + " " );
                out.write( "\n");
                //out.write( "]\n");
            }
            out.write( "]\n");
        }catch (java.io.IOException ex) {
            System.out.println(ex);
       }finally{
           if ( out != null )
               try{
                   out.close();
               }catch (java.io.IOException ex) {
                   System.out.println(ex);
               }
       }
    }

    public int getIdMapa() {
      return idMapa;
    }

    public void setIdMapa(int idMapa) {
      this.idMapa = idMapa;
    } 
}
