package PanelInterfaceMulticapa;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import Interface.MapCanvas;
import Interface.MapWindow;
import Interface.MultiCapas.InterfazMultiCapas;
import Map.Map;

public class PanelMultiCapas extends JPanel {

	private JDialog ventanaReg;
	private JButton botonEjecutar;
	private JButton botonReglas;
	private JButton botonSalir;
	private PanelMapaCargado mapa1;
	private PanelMapaCargado mapa2;
	private MapWindow mapWindow;
	private JOptionPane jop = new JOptionPane();
	private JTextField iteraciones;

	public InterfazMultiCapas interfazMultiCapas;

	public PanelMultiCapas(Boolean flag, MapWindow mapWindow) {
		mapa1 = new PanelMapaCargado("Mapa 1", flag, mapWindow);
		mapa2 = new PanelMapaCargado("Mapa 2", flag, mapWindow);
		this.mapWindow = mapWindow;
		inicializar();
		this.add(panelCentral());
		eventos();

	}

	private void inicializar() {
		interfazMultiCapas = new InterfazMultiCapas(mapWindow);
		iteraciones = new JTextField(4);
		iteraciones.setText("10");
		botonEjecutar = new JButton("Ejecutar");
		botonReglas = new JButton("Tipo de Reglas");
		botonSalir = new JButton("Salir");
	}

	private JPanel panelCentral() {
		JPanel panelAux = new JPanel();
		panelAux.setLayout(new BoxLayout(panelAux, BoxLayout.Y_AXIS));
		panelAux.add(panelSuperior());
		panelAux.add(panelInferior());
		return panelAux;
	}

	private JPanel panelSuperior() {
		JPanel panelAux = new JPanel();
		panelAux.add(mapa1);
		panelAux.add(new JLabel("      "));
		panelAux.add(mapa2);
		return panelAux;
	}

	private JPanel panelInferior() {
		JPanel panelAux = new JPanel();
		panelAux.setLayout(new BoxLayout(panelAux, BoxLayout.Y_AXIS));
		botonReglas.setAlignmentX(CENTER_ALIGNMENT);
		botonEjecutar.setAlignmentX(CENTER_ALIGNMENT);
		botonSalir.setAlignmentX(CENTER_ALIGNMENT);
		panelAux.add(botonReglas);
		panelAux.add(panelIteracion());
		panelAux.add(botonEjecutar);
		panelAux.add(botonSalir);
		return panelAux;
	}

	private JPanel panelIteracion() {
		JPanel panelAux = new JPanel();
		JLabel titulo = new JLabel("Maximo de Iteraciones:");
		panelAux.add(titulo);
		panelAux.add(iteraciones);
		return panelAux;
	}

	private void ventanaReglas() {
		Map m1 = mapa1.getMapCanvas().getMap();
		Map m2 = mapa2.getMapCanvas().getMap();
		ventanaReg = new JDialog(mapWindow, "Crear Reglas", true);
		ventanaReg.setSize(700, 600);
		ventanaReg.setLocationRelativeTo(null);
		ventanaReg.add(new PanelReglas(m1, m2, this));
		ventanaReg.setVisible(true);
	}

	private void iniciarSimulacion() {
		MapCanvas mapCanvas1 = mapa1.getMapCanvas();
		MapCanvas mapCanvas2 = mapa2.getMapCanvas();

		interfazMultiCapas.AsignarDatos(mapCanvas1, mapa1.getIteracion(),
				mapCanvas2, mapa2.getIteracion());
		interfazMultiCapas.asignarNormalizacion(mapa1.getNormalizacion(),
				mapa2.getNormalizacion());
		if (!iteraciones.getText().isEmpty()) {
			int maximoI = Integer.parseInt(iteraciones.getText());
			interfazMultiCapas.setMaximoIteraciones(maximoI);
		}

		interfazMultiCapas.Run();
	}

	private void eventos() {
		botonReglas.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				if (comprobarMapas())
					ventanaReglas();
			}
		});

		botonEjecutar.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				if (comprobarMapas() && comprobarIteraciones())
					iniciarSimulacion();
			}
		});

		botonSalir.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				mapWindow.cerrarVentana();
			}
		});
	}

	public JDialog getVentanaSubCon() {
		return ventanaReg;
	}

	private Boolean comprobarMapas() {
		if (!mapa1.getBandera() || !mapa2.getBandera()) {
			jop.showMessageDialog(this,
					"Deben estar cargado los 2 mapas para poder Continuar",
					"Error Mapas del sistema", JOptionPane.ERROR_MESSAGE);
			return false;
		}
		return true;
	}

	private Boolean comprobarIteraciones() {
		String cadena = iteraciones.getText();
		if (!cadena.matches("[0-9]*")) {
			jop.showMessageDialog(
					this,
					"El campo Maximo de Iteraciones debe estar conformados solo por digitos y deben ser positivos",
					"Campos invalidos", JOptionPane.ERROR_MESSAGE);
			return false;
		}
		return true;
	}

}
