package PanelInterfaceMulticapa;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import utilidades.ConvertirReglasConexion;
import utilidades.ConvertirReglasDifusa;
import utilidades.ConvertirReglasEcuaciones;

import Interface.MultiCapas.ReglasDifusas;
import Interface.MultiCapas.ReglasEcuacion;
import Map.Map;

@SuppressWarnings("serial")
public class PanelReglas extends JPanel {

	private JComboBox listaReglas;
	private Map mapa1;
	private Map mapa2;
	private PanelMultiCapas multiCapas;
	private PanelConstruccionRegla panelConexion;
	private PanelConstruccionRegla panelEcuacion;
	private PanelConstruccionRegla panelDifuzo;
	private JPanel panelTipoRegla;
	private JButton listo;
	private JLabel tituloActual;
	private JScrollPane scrollPane;

	public PanelReglas(Map mapa1, Map mapa2, PanelMultiCapas multiCapas) {
		this.mapa1 = mapa1;
		this.mapa2 = mapa2;
		this.multiCapas = multiCapas;
		inicializarPaneles();
		this.add(panelCentral());

	}

	private void llenarScroll() {
		scrollPane = new JScrollPane(panelTipoRegla);
		scrollPane.setPreferredSize(new Dimension(650, 450));
	}

	private void inicializarPaneles() {
		llenarComboBox();
		panelTipoRegla = new JPanel();
		panelConexion = new PanelConstruccionRegla(mapa1, mapa2, 0);
		panelEcuacion = new PanelConstruccionRegla(mapa1, mapa2, 1);
		panelDifuzo = new PanelConstruccionRegla(mapa1, mapa2, 2);
		listo = new JButton("Listo");
		panelTipoRegla.add(panelConexion);
		tituloActual = new JLabel("Tipo de regla actual: Conexion");
		llenarScroll();
	}

	private void llenarComboBox() {
		listaReglas = new JComboBox();
		listaReglas.addItem("Conexiones");
		listaReglas.addItem("Ecuaciones Matematicas");
		listaReglas.addItem("Reglas Difusas");
	}

	private JPanel panelCentral() {
		JPanel panelAux = new JPanel();
		panelAux.setLayout(new BoxLayout(panelAux, BoxLayout.Y_AXIS));
		JLabel titulo = new JLabel("Seleccione El tipo de Regla");
		titulo.setAlignmentX(CENTER_ALIGNMENT);
		tituloActual.setAlignmentX(CENTER_ALIGNMENT);
		listo.setAlignmentX(CENTER_ALIGNMENT);
		panelAux.add(titulo);
		panelAux.add(PanelTipoRegla());
		panelAux.add(new JLabel("    "));
		panelAux.add(tituloActual);
		panelAux.add(scrollPane);
		panelAux.add(listo);
		eventos();
		return panelAux;
	}

	private JPanel PanelTipoRegla() {
		JPanel panelAux = new JPanel();
		panelAux.add(listaReglas);
		return panelAux;
	}

	private void eventos() {

		listaReglas.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				Cambier();
			}
		});

		listo.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				if (validar()) {
					asignarReglas();
				}
			}
		});

	}

	protected void Cambier() {
		panelTipoRegla.removeAll();
		tituloActual.setText("Tipo de regla actual: "
				+ listaReglas.getSelectedItem());
		int m = listaReglas.getSelectedIndex();
		switch (m) {
		case 0:
			panelTipoRegla.add(panelConexion);
			break;

		case 1:
			panelTipoRegla.add(panelEcuacion);
			break;

		default:
			panelTipoRegla.add(panelDifuzo);
			break;
		}
		panelTipoRegla.updateUI();

	}

	private void asignarReglas() {
		int m = listaReglas.getSelectedIndex();
		switch (m) {
		case 0:
			ConvertirReglasConexion cRC = new ConvertirReglasConexion(
					panelConexion.getListaPanel());
			multiCapas.interfazMultiCapas.setListaReglasConexion(cRC
					.getReglasConexion());
			break;

		case 1:
			ConvertirReglasEcuaciones cRE = new ConvertirReglasEcuaciones(
					panelEcuacion.getListaPanel());
			multiCapas.interfazMultiCapas.setListaReglasEcuaciones(cRE
					.getReglasEcuacion());
			break;

		default:
			ConvertirReglasDifusa cRD = new ConvertirReglasDifusa(
					panelDifuzo.getListaPanel());
			multiCapas.interfazMultiCapas.setListaReglasDifusas(cRD
					.getReglasDifusa());
			break;
		}
		multiCapas.getVentanaSubCon().dispose();
	}

	private Boolean validar() {
		int m = listaReglas.getSelectedIndex();
		if (m == 0) {
			return validarReglasConexion();
		} else if (m == 1) {
			return validarReglasEcuacion();
		} else {
			return validarReglasDifusa();
		}
	}

	private Boolean validarReglasConexion() {
		ArrayList<JPanel> lPRC = panelConexion.getListaPanel();
		for (JPanel jP : lPRC) {
			PanelReglaConexion pRC=(PanelReglaConexion) jP;
			if(!pRC.validar(this))
				return false;
		}
		return true;
	}

	private Boolean validarReglasEcuacion() {
		ConvertirReglasEcuaciones cRE = new ConvertirReglasEcuaciones(
				panelEcuacion.getListaPanel());
		ArrayList<ReglasEcuacion> lRE = cRE.getReglasEcuacion();
		for (ReglasEcuacion rE : lRE) {
			rE.calcularPeso();
			if (rE.getError()) {
				rE.mostrarError(this);
				return false;
			}
		}
		return true;
	}

	private Boolean validarReglasDifusa() {
		ConvertirReglasDifusa cRD = new ConvertirReglasDifusa(
				panelDifuzo.getListaPanel());
		ArrayList<ReglasDifusas> lRD = cRD.getReglasDifusa();
		for (ReglasDifusas rD : lRD) {
			rD.calcularPeso();
			if (rD.getError()) {
				rD.mostrarError(this);
				return false;
			}
		}
		return true;
	}
}
