package Algorithms;

/**
 * RunFCM.java
 * Aqui se encuentran los algoritmos necesarios para la ejecucin de
 * Mapas Cognitivos Difusoso y Mapas Cognitivos Difusos Dinamicos
 * Created on 11 de febrero de 2005, 9:37
 * @author Jos Bladimir Contreras Vargas
 * @author <A HREF="mailto:jbc_ve@hotmail.com">jbc_ve@hotmail.com</A>
 */
public class RunFCM implements Runnable { 
    /**
     * Lienzo donde se debe dibujar el mapa
     */
    private Interface.MapCanvas mapCanvas;
    /**
     * Boton que ejecuta la ejecucion del mapa
     */
    private java.awt.Button play;
    /**
     * Hilo de ejecucion utilizado para el control del retardo
     */
    private java.lang.Thread hilo;
    /**
     * Etiqueta donde se muestra la iteracion actual
     */
    private java.awt.Label etiqueta;
    
    /**
     * Constante para indicar la Normalizacion del Tipo SY
     */
    public static final int SY = 1;
    /**
     * Constante para indicar la Normalizacion del Tipo SCY
     */
    public static final int SYC = 2;
    /**
     * Constante para indicar la Normalizacion del Tipo TriEstate
     */
    public static final int TRISTATE = 3;
    /**
     * Constante para indicar la Normalizacion del Tipo BiState
     */
    public static final int BISTATE = 4;
    
    /**
     * Variable para indicar el Maximo de Iteraciones
     */
    private int MaxIteraciones = 10;
    /**
     * Variable para indicar si el Sistema debe parar al estabilizar
     * falso por defecto
     */
    public boolean pararPorEstabilizar = false;
    
    /**
     * Variable para determinar que tipo de satuaracin se debe utilizar
     * se deben utilizar las constantes RunFCM.SCY, RunFCM.SY, RunFCM.BISTATE o RunFCM.TRISTATE
     */
    public int tipoSaturacion = 2;
    
    /**
     * Variable para indicar el retardo durante la ejecucion
     */
    private int retardo = 0;
    
    /**
     * Variable de computo de las reglas en el caso de mapas dinamicos
     */
    private Algorithms.Rules rules = new Algorithms.Rules();
    
    /**
     * Asigna el Maximo de Iteraciones a ejecutar por defecto son 10
     * @param n Numero de Iteraciones
     */        
    public void setMaxIteraciones ( int n ){
        if ( n < 2 )
            return;
        this.MaxIteraciones = n;
    }
    
    /**
     * Obtiene el maximo de iteracion en la ejecucion del mapa
     * @return Obtener el Maximo de Iteraciones Asignadas
     */
    public int getMaxIteraciones (){
        return this.MaxIteraciones;
    }
    
    /**
     * devuelve cierto si el mapa estabilizo
     * @return Devuelve cierto si el sistema ya estabilizo
     */
    private boolean Estabilizo(){
        if ( this.mapCanvas.getMap().sizeConceptsData() <= 1 )
            return false;
        
        Map.Concept ref;
        for ( int i = 0 ; i < this.mapCanvas.getMap().sizeConceptsData()-1 ; i++){
            java.util.Iterator <Map.Concept> it = this.mapCanvas.getMap().getConceptList().iterator();
            while (  it.hasNext()  ){
                ref = it.next();
                if ( Math.abs(ref.getValue(i) - ref.getCurrentValue() ) > 0.00001 )
                    break;
                else if ( !it.hasNext() ){
                    System.out.println("Estabilidad en los estados " + i + " al " + (this.mapCanvas.getMap().sizeConceptsData()-1) );
                    return true;
                }
            }
        }
        return false;
    }
    
    /**
     * Indica si el mapa se debe continuar ejecutando o no
     * @param n Numero de la Iteracin Actual
     * @return Cierto si debe Continuar Iterando falso en caso contrario
     */
    private boolean Continue (int n){
        if ( n >= this.MaxIteraciones )
            return false;
        if ( this.pararPorEstabilizar )
            return !Estabilizo();
        return true;
    }
    
    /**
     * Crea un nuevo hilo para la ejecucion del mapa
     * @param mapa Mapa que se va a ejecutar
     * @param canvas Lienzo donde se va a dibujar el mapa
     * @param play Boton que llama a la ejecucin del mapa
     * @param iteracion Etiqueta donde se imprime las iteraciones
     */
    public RunFCM( Interface.MapCanvas mapCanvas , java.awt.Button play , java.awt.Label iteracion) {
        this.mapCanvas = mapCanvas;
        this.play = play;
        this.etiqueta = iteracion; 
        hilo = new java.lang.Thread ( this , "Ejecucion" );
    }
  
    /**
     * Asigana un retardo a la ejecuion
     * @param n Retardo en la ejecucin en Mili Segundos
     */
    public void setRetardo (int n){
        if ( n < 0 )
            return;
        this.retardo = n;
    }
    
    /**
     * Retorna el retardo
     * @return Retardo en milisegundos
     */
    public int getRetardo(){
        return this.retardo;
    }
    
    /**
     * Normalizacion S(y) = 1/(1+exp( -9*(y-0.5) ) )
     * @param y Valor a Normalizar
     * @return Valor Normalizado
     */
    private float Sy ( float y ){         
        return (float) (1.0/(1.0+Math.exp( -9.0 * (y-0.5) )));
    }
    
    /**
     * Normalizacion por saturacion 
     * if v > 1 then v = 1 
     * else if v < 0 then v = 0
     * @param y Valor a Normalizar
     * @return Valor Normalizado
     */
    private float SyC ( float y ){
        if ( y < 0 )
            return 0.0f;
        else if ( y > 1 )
            return 1.0f;
        else
            return y;
    }
    
    /**
     *Normalizacion que permite dos estados
     *0 si y <= 0.5
     *1 si y > 1
     * @param y Valor a Normalizar
     * @return Valor Normalizado
     */
    private float BiState ( float y ){
        if ( y <= 0.5f )
            return 0.0f;
        return 1.0f;
    }
    
    /**
     *Normalizacion de tres estados
     *0 si y <= 1/3
     *0.5 si y > 1/3 y y <= 2/3
     *1 si y > 2/3
     * @param y Valor a Normalizar
     * @return Valor Normalizado
     */
    private float TriState ( float y ){
        if ( y <= 1.0 / 3.0 )
            return 0.0f;
        else if ( y <= 2.0/3.0 )
            return 0.5f;
                    
        return 1.0f;
    }
    
    /**
     * Normalizacion segun la seleccion utilizada por la interfaz grafica
     * estas pueden ser SCy Sy TriState o Bistate
     * @param y Valor a normalizar
     * @return Valor Normalizado
     */
    private float normalization ( float y ){
        if (this.tipoSaturacion == this.BISTATE )
            return this.BiState(y);
        
        else if (this.tipoSaturacion == this.TRISTATE )
            return this.TriState(y);
        
        else if (this.tipoSaturacion == this.SY )
            return this.Sy(y);
        
        else //if (this.tipoSaturacion == this.SYC )
            return this.SyC(y);   
    }
    
    /**
     * Punto de Ejecucin del Hilo aqui se determina si el mapa se debe ejecutar como un mapa dinamico o estatico
     */
    public void run (){
        this.play.setEnabled( false );
        
        if ( this.mapCanvas.getMap().isDynamic() )
            this.runDynamic();
        else
            this.runStatic();
        
        this.play.setLabel("Reset");
        this.play.setEnabled( true );
    }
    
    /**
     * Ejecucion del mapa como un mapa Statico
     */
    private void runStatic(){
        int c=1;
        do{
            java.util.Iterator <Map.Relation> itE = this.mapCanvas.getMap().getRelationList().iterator();
            while (  itE.hasNext()  ){
                Map.Relation ref = itE.next();
                ref.getFinalConcept().valorAuxiliar += ref.getInitialConcept().getLastValue()*ref.getValue();
            }
            java.util.Iterator <Map.Concept> itN = this.mapCanvas.getMap().getConceptList().iterator();
            while ( itN.hasNext() ){
                Map.Concept ref = itN.next();
                ref.addLastValue(  this.normalization( ref.valorAuxiliar ) );
                ref.setCurrentValue( ref.getLastValue() );
                ref.valorAuxiliar=0.0f;
            }
            
            this.mapCanvas.update( this.mapCanvas.getGraphics() );
            
            try{
                this.hilo.sleep(this.retardo);
            }catch ( java.lang.InterruptedException e ){
                System.out.println ( e );
            }
            this.mapCanvas.getMap().setCurrentIteration( c++ );
            this.etiqueta.setText( this.mapCanvas.getMap().getIterationLabel() );
        }while ( this.Continue( c ) );

    }

    /**
     * Ejecucion del mapa como un mapa Dynamico
     */
    private void runDynamic(){
        int c=1;
        float valor;
        do{
            java.util.Iterator <Map.Relation> itE = this.mapCanvas.getMap().getRelationList().iterator();
            while (  itE.hasNext()  ){
                Map.Relation ref = itE.next();
                this.rules.computeRelationValue( ref );
                valor = Math.abs( ref.getInitialConcept().getLastValue() );
                //ref.getFinalConcept().valorAuxiliar += ( 0.5f*valor+0.5f )*ref.getValue();
                ref.getFinalConcept().valorAuxiliar += valor*ref.getValue();
            }
            java.util.Iterator <Map.Concept> itN = this.mapCanvas.getMap().getConceptList().iterator();
            while ( itN.hasNext() ){
                Map.Concept ref = itN.next();
                ref.addLastValue(  this.normalization( ref.valorAuxiliar ) );
                ref.setCurrentValue( ref.getLastValue() );
                ref.valorAuxiliar=0.0f;
            }
            
            this.mapCanvas.update( this.mapCanvas.getGraphics() );
            
            try{
                this.hilo.sleep(this.retardo);
            }catch ( java.lang.InterruptedException e ){
                System.out.println ( e );
            }
            this.mapCanvas.getMap().setCurrentIteration( c++ );
            this.etiqueta.setText( this.mapCanvas.getMap().getIterationLabel() );
        }while ( this.Continue( c ) );
    }
}
