package Interface;

/**
 * Este Objeto se Utiliza para agregar Componentes a un panel utilizando GridBagLayout
 * Created on 25 de enero de 2005, 21:05
 * @author Jos Bladimir Contreras Vargas
 * @author <A HREF="mailto:jbc_ve@hotmail.com">jbc_ve@hotmail.com</A>
 */
public class LPanel extends java.awt.Panel {
    //Variable Utilizada`para los espacios entre los componentes
    /**
     * Indica el espacio entre los componenets por defecto es de 8
     */
    public int spacing = 8; 
    //layout que no nesetita que todos sus elementos tengan el mismo tamao
    /**
     * manejador del layaout
     */
    public java.awt.GridBagLayout layout;    
    //indica como y donde colocar un elemento en el layout
    /**
     * Manejador de los parametros para acomodar los componentes
     */
    public java.awt.GridBagConstraints constraints;
    
    /**
     * Crea Una Nueva Instancia de LPanel sin componentes
     */
    public LPanel(){
        super();
      
        layout = new java.awt.GridBagLayout();
        constraints = new java.awt.GridBagConstraints();
        this.setLayout(layout);
      
        //indica el espacio minimo entre el borde y los componentes
        constraints.insets = new java.awt.Insets(spacing, spacing, spacing, spacing);
        
        //que los componentes no se estires
        constraints.fill = java.awt.GridBagConstraints.NONE;
        
        //espacio entre componentes cero
        constraints.ipadx = constraints.ipady = 0;
        
        //el espacio vacio se debe estirar en x y y por igual
        constraints.weightx = constraints.weighty = 1.0;
    }
     
    /**
     * Agrega un Componente al Panel segn los parametros especificados
     * @param component Componente a agregar
     * @param width cuantas celdas de ancho ocupa 0 ocupa todo el espacio horizontal
     * @param anchor Negativo a la izquierda cero(0) al centro Positivo a la derecha
     * @param weightx determina como distribuir el espacio sobrante Horizontal
     * @param weighty determina como distribuir el espacio sobrante Vertical
     * @param fill espesifica como redibujar el componente 0 No deformar 1 Horizontalmente 2 Verticalmente >2 en ambos sentidos
     * @param shift 1 o 3 determina espacio izquierdo cero 2 o 3 determina espacio derecho cero
     * @return Retorna el componente agregado
     */
    public java.awt.Component addComponent (java.awt.Component component, int width,
                                    int anchor, double weightx, double weighty, int fill,
                                    int shift) {
        //espacio a la izquierda 
        constraints.insets.left = spacing;
        //espacio a la derecha
        constraints.insets.right = spacing;
         
        if(shift == 1 || shift == 3)
            constraints.insets.left = 0;
        
        if(shift == 2 || shift == 3)
            constraints.insets.right = 0;
      
        if(anchor < 0)
            constraints.anchor = java.awt.GridBagConstraints.WEST;
        else if(anchor == 0)
            constraints.anchor = java.awt.GridBagConstraints.CENTER;
        else
            constraints.anchor = java.awt.GridBagConstraints.EAST;
      
        if(width == 0)
            constraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        else
            constraints.gridwidth = width;
      
        constraints.weightx = weightx;
        constraints.weighty = weighty;
      
        if(fill == 0)
            constraints.fill = java.awt.GridBagConstraints.NONE;
        else if(fill == 1)
            constraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        else if(fill == 2)
            constraints.fill = java.awt.GridBagConstraints.VERTICAL;
        else
            constraints.fill = java.awt.GridBagConstraints.BOTH;
      
        layout.setConstraints(component, constraints);
        this.add(component);
      
        if(width == 0)
           constraints.insets.top = constraints.insets.bottom = spacing;
      
        return component;
      }  
}
