package Interface;

/**
 * Lienzo donde dibujar el mapa
 * Created on 25 de enero de 2005, 17:22
 * @author Jos Bladimir Contreras Vargas
 * @author <A HREF="mailto:jbc_ve@hotmail.com">jbc_ve@hotmail.com</A>
 */
public class MapCanvas extends java.awt.Canvas { 
    
    /**
     * Constante para indicar que se esta en Modo de crear Conceptos
     */
    public static final int CREATE_CONCEPTS = 0;
    /**
     * Constante para indicar que se esta en Modo de crear Relaciones
     */
    public static final int CREATE_RELATIONS = 1;
    /**
     * Constante para indicar que se esta en Modo de selecionar Conceptos
     */
    public static final int SELECT_CONCEPTS = 2;
    /**
     * Constante para indicar que se esta en Modo de selecionar Relaciones
     */
    public static final int SELECT_RELATIONS = 3;
    /**
     * Constante para indicar que se esta en Modo de eliminar Conceptos
     */
    public static final int DELETE_CONCEPTS = 4;
    /**
     * Constante para indicar que se esta en Modo de eliminar Relaciones
     */
    public static final int DELETE_RELATIONS = 5; 
    /**
     * Mapa a dibujar en el lienzo
     */
    private Map.Map map;
    /**
     * inidca si los nodos se deben dibujar con un tamao proporcional al valor de los mismos
     */
    public boolean dibujarPorpocionalmente = false;  
    /**
     * Variable para indicar si actualmente se esta creando una relacin
     */
    public boolean creatingRelation = false;
    /**
     * Concepto utilizado durante el modo de crear relaciones, mientras una relacion no tenga un concepto final asignado
     */
    public Map.Concept startingConcept;
    /**
     * Variable para identificar la posicion del raton
     */
    public java.awt.Point mousePoint = new java.awt.Point (0,0);   
    /**
     * Variable para indicar el modo del ratn, se debe utilizar las constantes:
     * <PRE>MapCanvas.CREATE_CONCEPTS, MapCanvas.SELECT_CONCEPTS , MapCanvas.DELETE_CONCEPTS
     * MapCanvas.CREATE_RELATIONS, MapCanvas.SELECT_RELATIONS , MapCanvas.DELETE_RELATIONS</PRE>
     */
    private int	mouseMode = CREATE_CONCEPTS;
    
    /**
     * Crea un nuevo lienzo para diujar los grafos
     * @param map_in grafo de entrada
     */
    public MapCanvas(Map.Map map_in){
         this.map = map_in;
         this.setBackground(java.awt.Color.yellow);
         this.setFont(new java.awt.Font("Microsoft Sans Serif", 1, 16)); // Para cambiar el tamao de fuente del canvas
      }

    /**
     * Funcion que dibuja el mapa en el lienzo
     * @param g Objeto grafico donde se dibujara el mapa
     */
    public void paint( java.awt.Graphics g ){
        this.map.drawMap( g , dibujarPorpocionalmente );
        
        if ( this.creatingRelation ){
            java.awt.Point center = this.startingConcept.getPosition();
            double angle = Math.atan2( center.y - this.mousePoint.y , center.x - this.mousePoint.x );
            java.awt.Point p1 = new java.awt.Point ( (int)Math.round( center.x -  this.startingConcept.getRadio() * Math.cos(angle) + this.startingConcept.getRadio() ) , 
                                   (int)Math.round( center.y -  this.startingConcept.getRadio() * Math.sin(angle) + this.startingConcept.getRadio() )  );
            g.drawLine( p1.x , p1.y , this.mousePoint.x , this.mousePoint.y );
        }
    }
     
    /**
     * Obtiene el modo actual del raton
     * @return valores Posibles:
     * <PRE>MapCanvas.CREATE_CONCEPTS, MapCanvas.SELECT_CONCEPTS , MapCanvas.DELETE_CONCEPTS
     * MapCanvas.CREATE_RELATIONS, MapCanvas.SELECT_RELATIONS , MapCanvas.DELETE_RELATIONS</PRE>
     */
    public int getMouseMode ( ){
        return mouseMode;
    }
     
    /**
     * Asigna el Modo de Accion del raton
     * @param mode Valores Posibles 
     * <PRE>MapCanvas.CREATE_CONCEPTS, MapCanvas.SELECT_CONCEPTS , MapCanvas.DELETE_CONCEPTS
     * MapCanvas.CREATE_RELATIONS, MapCanvas.SELECT_RELATIONS , MapCanvas.DELETE_RELATIONS</PRE>
     */
    public void setMouseMode(int mode){
         this.mouseMode = mode;
    }
    
    public Map.Map getMap (){
        return this.map;
    }
}
