package Interface;

import Interface.Options.DelayOptions;
import Interface.Options.ConceptOptions;
import Interface.Options.MaxIterationOptions;
import Interface.Options.RelationOptions;

/**
 * Ventana para la ejecucin y analisis de Mapas Cognitivos
 * Created on 25 de enero de 2005, 10:36
 * @author Jos Bladimir Contreras Vargas
 * @author <A HREF="mailto:jbc_ve@hotmail.com">jbc_ve@hotmail.com</A>
 */
public class MapWindow extends javax.swing.JFrame {
     /**
      * Lienzo donde se dibuja el mapa
      */
     private MapCanvas mapCanvas;
     /**
      * Hilo para la ejecucin del mapa
      */
     private Algorithms.RunFCM hilo;
     /**
      * Etiqueta para mostar la iteracin Actual
      */
     private java.awt.Label iteracion = new java.awt.Label ("0001 / 0001");
     /**
      * Concepto a arrastrar con el raton
      */
     private Map.Concept conceptToDrag = null;

    /**
     * Crea una nueva ventana
     * @param dynamic Indica si el Mapa es Dinamico o no
     * true - dinamico
     * false - statico
     */
    public MapWindow(boolean dynamic){
        construct( new Map.Map(dynamic) );
    }
    
    /**
     * Funcio que crea la ventana
     * @param map_in Mapa para la ventana
     */
    private void construct(Map.Map map_in){
        
        this.mapCanvas = new MapCanvas( map_in );
        this.mapCanvas.setBounds(0,0, 1024,1024);
        
        
        this.mapCanvas.addMouseListener( new AccionesRatonEnMapCanvas () );
        
        this.setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        this.setLayout(new java.awt.BorderLayout());
        
        this.setMenuBar( this.MakeMenuBar() );
        this.add( this.MakeLPanel( ) , java.awt.BorderLayout.WEST );
        
      	// the map viewing panel (canvas and scrollbars)
        java.awt.ScrollPane scp = new java.awt.ScrollPane();
        scp.add( this.mapCanvas );
        this.add( scp , java.awt.BorderLayout.CENTER );
        
        this.setLocation( 50 , 50 );
        this.validate();
    }
    
    /**
     * Funcion que crea el panel Izquierdo
     * @return Retorna el panel creado
     */
    private java.awt.Panel MakeLPanel(){
        java.awt.Label posicionRaton = new java.awt.Label("X: 0.0 Y:0.0");
        this.mapCanvas.addMouseMotionListener( new MovimientosRatonEnMapCanvas( posicionRaton ) );
        
        //Panel Izquierdo para control
        Interface.LPanel PanelIzquierdo = new Interface.LPanel();
        PanelIzquierdo.setBackground( java.awt.Color.LIGHT_GRAY);
        PanelIzquierdo.addComponent( posicionRaton, 0, 1, 1.0, 0.0, 1, 0 );  
        java.awt.CheckboxGroup mode_group = new java.awt.CheckboxGroup();
        PanelIzquierdo.addComponent( new javax.swing.JLabel ("Acciones del Ratn") , 0 , -1 , 1.0 , 1.0 , 0 , 0); 
        CambiaAccionesDelRaton escuchaCheckBox = new CambiaAccionesDelRaton();
        PanelIzquierdo.constraints.insets.top = PanelIzquierdo.constraints.insets.bottom = 0;
        java.awt.Checkbox cTemp = new java.awt.Checkbox ("Crear Conceptos", mode_group, true);
        cTemp.setName("CN");
        cTemp.addItemListener( escuchaCheckBox );
        PanelIzquierdo.addComponent( cTemp , 0, -1, 1.0, 1.0, 0, 0);
         
        PanelIzquierdo.constraints.insets.top = PanelIzquierdo.constraints.insets.bottom = 0;
        cTemp = new java.awt.Checkbox ("Crear Relaciones", mode_group, false);
        cTemp.setName("CE");
        cTemp.addItemListener( escuchaCheckBox );
        PanelIzquierdo.addComponent( cTemp , 0 , -1, 1.0, 1.0, 0, 0);   
         
        PanelIzquierdo.constraints.insets.top = PanelIzquierdo.constraints.insets.bottom = 0;
        cTemp = new java.awt.Checkbox ("Seleccionar Conceptos", mode_group, false);
        cTemp.setName("SN");
        cTemp.addItemListener( escuchaCheckBox );
        PanelIzquierdo.addComponent( cTemp , 0 , -1, 1.0, 1.0, 0, 0);
        
        PanelIzquierdo.constraints.insets.top = PanelIzquierdo.constraints.insets.bottom = 0;
        cTemp = new java.awt.Checkbox ("Seleccionar Relaciones", mode_group, false);
        cTemp.setName("SE");
        cTemp.addItemListener( escuchaCheckBox );
        PanelIzquierdo.addComponent( cTemp, 0, -1, 1.0, 1.0, 0, 0);
        
        PanelIzquierdo.constraints.insets.top = PanelIzquierdo.constraints.insets.bottom = 0;
        cTemp = new java.awt.Checkbox ("Eliminar Conceptos", mode_group, false);
        cTemp.setName("DN");
        cTemp.addItemListener( escuchaCheckBox );
        PanelIzquierdo.addComponent( cTemp , 0 , -1, 1.0, 1.0, 0, 0);

        PanelIzquierdo.constraints.insets.top = PanelIzquierdo.constraints.insets.bottom = 0;
        cTemp = new java.awt.Checkbox ("Eliminar Relaciones", mode_group, false);
        cTemp.setName("DE");
        cTemp.addItemListener( escuchaCheckBox );
        PanelIzquierdo.addComponent( cTemp, 0, -1, 1.0, 1.0, 0, 0);     

        PanelIzquierdo.addComponent( new javax.swing.JLabel ("Ejecucin:") , 0 , -1 , 1.0 , 1.0 , 0 , 0);
        
        PanelIzquierdo.spacing = 8;
        PanelIzquierdo.constraints.insets.top = PanelIzquierdo.constraints.insets.bottom = 0;
        
        java.awt.Button play = new java.awt.Button ("Play");
        java.awt.TextField entradaIteracion = new java.awt.TextField ("0001");
        PanelIzquierdo.addComponent ( play , 0, 0, 1.0, 1.0, 0, 0);
        play.addMouseListener( new EscuchaBotonPLay (entradaIteracion) );
      
        PanelIzquierdo.addComponent( new javax.swing.JLabel ("Visualizar:") , 0 , -1 , 1.0 , 1.0 , 0 , 0);
        PanelIzquierdo.addComponent( new javax.swing.JLabel ("Iteracion") , 1 , -1 , 1.0 , 1.0 , 0 , 0);
        PanelIzquierdo.addComponent( this.iteracion , 0 , -1 , 1.0 , 1.0 , 0 , 0);
        PanelIzquierdo.constraints.insets.top = PanelIzquierdo.constraints.insets.bottom = 0;
        PanelIzquierdo.addComponent ( entradaIteracion , 0 , 0 , 1.0 , 1.0 , 0 , 0 );
        
        Interface.LPanel panelDeIteracion = new Interface.LPanel();
        
        EscuchaVistaIteracion escucha1 = new EscuchaVistaIteracion( entradaIteracion );
        java.awt.Button bTemp = new java.awt.Button ("<<");
        bTemp.addMouseListener(escucha1);
        panelDeIteracion.addComponent (  bTemp , 1 , 1 , 1.0 , 1.0 , 0 , 0 );
        
        bTemp = new java.awt.Button ("Ver");
        bTemp.addMouseListener(escucha1);
        panelDeIteracion.addComponent ( bTemp , 1 , 0 , 1.0 , 1.0 , 0 , 0 );
        
        bTemp = new java.awt.Button (">>");
        bTemp.addMouseListener(escucha1);
        panelDeIteracion.addComponent ( bTemp  , 0 , -1 , 1.0 , 1.0 , 0 , 0 );
        
       
        PanelIzquierdo.constraints.insets.top = PanelIzquierdo.constraints.insets.bottom = 0;
        PanelIzquierdo.addComponent ( panelDeIteracion , 0 , 0 , 1.0 , 1.0 , 0 , 0 );
        
        this.hilo = new Algorithms.RunFCM ( this.mapCanvas, play , this.iteracion );
        java.awt.Panel controls_superpanel = new java.awt.Panel();
      
        controls_superpanel.add("North", PanelIzquierdo);
        controls_superpanel.setBackground( java.awt.Color.LIGHT_GRAY);
        return controls_superpanel;
    }
 
    /**
     * Funcion que crea la barra de menu
     * @return Retorna la barra de menu creada
     */
    private java.awt.MenuBar MakeMenuBar (){
        java.awt.MenuBar barraDeMenu = new java.awt.MenuBar();
        EscuchaMenuAbrir escucha = new EscuchaMenuAbrir();
        java.awt.Menu m = new java.awt.Menu( "Archivo" );
        
        java.awt.MenuItem mi = new java.awt.MenuItem ( "Abrir" );
        m.add( mi );
        mi.addActionListener( escucha );
        mi = new java.awt.MenuItem ( "Guardar" ); 
        m.add( mi );
        mi.addActionListener( escucha );
        mi = new java.awt.MenuItem ( "Guardar Ejecucin" );
        m.add( mi );
        mi.addActionListener( escucha );
        m.addSeparator();
        mi = new java.awt.MenuItem ( "Nuevo" );
        m.add( mi );
        mi.addActionListener( escucha );
        m.addSeparator();
        mi = new java.awt.MenuItem ( "Salir" );
        m.add ( mi );
        mi.addActionListener( escucha );
        barraDeMenu.add ( m );
        
        m = new java.awt.Menu ("Vista");
        java.awt.CheckboxMenuItem mc1 = new java.awt.CheckboxMenuItem ("Proporcional", false);
        m.add( mc1 );
        
        java.awt.CheckboxMenuItem mc2 = new java.awt.CheckboxMenuItem ("Fijo", true );
        m.add( mc2 );
        
        EscuchaMenuVista escuchaVista = new EscuchaMenuVista(mc1 , mc2);
        
        mc1.addItemListener( escuchaVista );
        mc2.addItemListener( escuchaVista );
        
        barraDeMenu.add ( m );
        
        m = new java.awt.Menu ("Ejecucin");
        mi = new java.awt.MenuItem ( "Asignar Retardo" );
        m.add(mi);
        
        m.addSeparator();
        
        m.add(new java.awt.MenuItem ( "Asignar Normalizacin" ));
        mc1 = new java.awt.CheckboxMenuItem ("      BiState", false);
        mc2 = new java.awt.CheckboxMenuItem ("      TriState", false);
        java.awt.CheckboxMenuItem mc3 = new java.awt.CheckboxMenuItem ("      Saturacion", true);
        java.awt.CheckboxMenuItem mc4 = new java.awt.CheckboxMenuItem ("      Sy", false);
        m.add ( mc1);
        m.add ( mc2);
        m.add ( mc3);
        m.add ( mc4);
        EscuchaMenuEjecucion ee = new EscuchaMenuEjecucion ( mc1,mc2,mc3,mc4);
        mc1.addItemListener( ee );
        mc2.addItemListener( ee );
        mc3.addItemListener( ee );
        mc4.addItemListener( ee );
        mi.addActionListener( new java.awt.event.ActionListener(){
             public void actionPerformed ( java.awt.event.ActionEvent ae ){
                 new DelayOptions(MapWindow.this.hilo);
             }
        });
        
        m.addSeparator();
        
        mc1 = new java.awt.CheckboxMenuItem ("Parar al estabilizar", false);
        mc1.addItemListener( new Estabiliza() ); 
        m.add(mc1);
        mi = new java.awt.MenuItem ( "Asignar Maximo de Iteraciones" );
        mi.addActionListener( new java.awt.event.ActionListener(){
             public void actionPerformed ( java.awt.event.ActionEvent ae ){
                 new MaxIterationOptions(MapWindow.this.hilo);
             }
        });
        m.add(mi);
        
        barraDeMenu.add ( m );
        
        return barraDeMenu;
    }
    
    /**
     * India si el la ejecucin del mapa debe para al estabilizar o no
     */
    private class Estabiliza implements java.awt.event.ItemListener{  
        /**
         * Funcion que se activa al cambiar el estado del item
         * @param e evento que se lanza al cambiar el estado del item
         */
        public void itemStateChanged( java.awt.event.ItemEvent e){
            MapWindow.this.hilo.pararPorEstabilizar = (e.getStateChange() == e.SELECTED); 
        }
    }
    
    /**
     *Escucha para indicar que modo del raton hay actualmente
     */
    private class CambiaAccionesDelRaton implements java.awt.event.ItemListener{  
        /**
         * Funcion que se lanza al cambiar el Estado del item
         * @param e que se ejecuta la cambiar el estado
         */
        public void itemStateChanged( java.awt.event.ItemEvent e){
            
            if (((java.awt.Checkbox)e.getSource()).getName().equals("CN"))
                MapWindow.this.mapCanvas.setMouseMode( mapCanvas.CREATE_CONCEPTS );
            
            if (((java.awt.Checkbox)e.getSource()).getName().equals("DN"))
                MapWindow.this.mapCanvas.setMouseMode( mapCanvas.DELETE_CONCEPTS );
            
            else if (((java.awt.Checkbox)e.getSource()).getName().equals("SN"))
                MapWindow.this.mapCanvas.setMouseMode( mapCanvas.SELECT_CONCEPTS );

            else if (((java.awt.Checkbox)e.getSource()).getName().equals("CE"))
                MapWindow.this.mapCanvas.setMouseMode( mapCanvas.CREATE_RELATIONS );

            else if (((java.awt.Checkbox)e.getSource()).getName().equals("SE"))
                MapWindow.this.mapCanvas.setMouseMode( mapCanvas.SELECT_RELATIONS );

            else if (((java.awt.Checkbox)e.getSource()).getName().equals("DE"))
                MapWindow.this.mapCanvas.setMouseMode( mapCanvas.DELETE_RELATIONS );
       
        }
    }
    
    /**
     * Escucha para Indicar la posicion del raton sobre el lienzo, o para arrastrar algun concepto
     */
    private class MovimientosRatonEnMapCanvas implements java.awt.event.MouseMotionListener{ 
        private java.awt.Label posicionRaton;
        
        public MovimientosRatonEnMapCanvas ( java.awt.Label posicion ){
            this.posicionRaton = posicion;
        }
        
        /**
         * Evento del moviento del raton sobre el lienzo
         * @param e evento de mover el raton
         */
        public void mouseMoved ( java.awt.event.MouseEvent e){
            this.posicionRaton.setText( "X: "+ e.getX() + " Y: " + e.getY() );
            if ( MapWindow.this.mapCanvas.creatingRelation ){
                MapWindow.this.mapCanvas.mousePoint.setLocation( e.getX() , e.getY() );
                MapWindow.this.mapCanvas.repaint();
            }
        }
        
        /**
         * evento de arrastrar el raton en el lienzo
         * @param e evento de arrastrar el raton
         */
        public void mouseDragged  ( java.awt.event.MouseEvent e ){
            this.posicionRaton.setText( "X: "+ e.getX() + " Y: " + e.getY() );
            java.awt.Point raton = new java.awt.Point ( e.getX() , e.getY() );       
            if (MapWindow.this.mapCanvas.getMouseMode() == MapWindow.this.mapCanvas.SELECT_CONCEPTS ){
                if ( MapWindow.this.conceptToDrag == null )
                    MapWindow.this.conceptToDrag = MapWindow.this.mapCanvas.getMap().findConcept( raton.x , raton.y );
                
                if ( MapWindow.this.conceptToDrag != null ){
                    MapWindow.this.conceptToDrag.setPosition( raton.x - MapWindow.this.conceptToDrag.getRadio() , raton.y - MapWindow.this.conceptToDrag.getRadio() );
                    MapWindow.this.mapCanvas.repaint();
                }
            }
        }
    }
    
    /**
     * Escucha para las acciones de dibujar
     */
    private class AccionesRatonEnMapCanvas extends java.awt.event.MouseAdapter{ 
        /**
         * evento de soltar el raton al soltar el raton se elimina el concepto a arrastrar
         * @param e evento del raton
         */
        public void mouseReleased( java.awt.event.MouseEvent e) {        
            if ( MapWindow.this.conceptToDrag != null )
                MapWindow.this.conceptToDrag = null;
        } 
  
        /**
         * Evento de cliquear sobre el lienzo
         * @param e evento del raton
         */
        public void mouseClicked( java.awt.event.MouseEvent e) {
            java.awt.Point raton = new java.awt.Point ( e.getX() , e.getY() );
            
            /**************************************/
            /********CREAR CONCEPTOS O EJES********/
            /**************************************/
            if ( MapWindow.this.mapCanvas.getMouseMode() == MapWindow.this.mapCanvas.CREATE_CONCEPTS && e.getButton() == e.BUTTON1  ){
                MapWindow.this.mapCanvas.getMap().addConcept( raton.x  , raton.y , MapWindow.this.mapCanvas.getMap().isDynamic() );
                MapWindow.this.mapCanvas.repaint();
            }
            else if ( MapWindow.this.mapCanvas.getMouseMode() == MapWindow.this.mapCanvas.CREATE_RELATIONS && e.getButton() == e.BUTTON1 ){
                Map.Concept selected = MapWindow.this.mapCanvas.getMap().findConcept( raton.x , raton.y );
                if ( selected != null && ! MapWindow.this.mapCanvas.creatingRelation ){
                    MapWindow.this.mapCanvas.startingConcept = selected;
                    MapWindow.this.mapCanvas.creatingRelation = true;
                }
                else if ( selected != null && MapWindow.this.mapCanvas.creatingRelation ){
                    MapWindow.this.mapCanvas.getMap().addRelation( MapWindow.this.mapCanvas.startingConcept , selected );
                    MapWindow.this.mapCanvas.repaint();
                    MapWindow.this.mapCanvas.startingConcept = null;
                    MapWindow.this.mapCanvas.creatingRelation = false;
                }
            }
            /**************************************/
            /*****ELIMINAR CONCEPTOS O EJES********/
            /**************************************/
            else if ( MapWindow.this.mapCanvas.getMouseMode() == MapWindow.this.mapCanvas.DELETE_CONCEPTS ){
                Map.Concept selected = MapWindow.this.mapCanvas.getMap().findConcept( e.getX() , e.getY() );
                if ( selected != null ){
                    MapWindow.this.mapCanvas.getMap().deleteConcept ( selected ); 
                    MapWindow.this.mapCanvas.repaint();
                }
            }
            else if ( MapWindow.this.mapCanvas.getMouseMode() == MapWindow.this.mapCanvas.DELETE_RELATIONS ){
                Map.Relation selected = MapWindow.this.mapCanvas.getMap().findRelation ( e.getX() , e.getY() );
                if ( selected != null ){
                    MapWindow.this.mapCanvas.getMap().deleteRelation ( selected ); 
                    MapWindow.this.mapCanvas.repaint();
                }
            }
            
            /**************************************/
            /*****OPCIONES CONCEPTOS O EJES********/
            /**************************************/
            else if ( (MapWindow.this.mapCanvas.getMouseMode() == MapWindow.this.mapCanvas.SELECT_CONCEPTS || MapWindow.this.mapCanvas.getMouseMode() == MapWindow.this.mapCanvas.CREATE_CONCEPTS) && ( e.getButton() == e.BUTTON2 || e.getButton() == e.BUTTON3 ) ){
                Map.Concept selected = MapWindow.this.mapCanvas.getMap().findConcept( e.getX() , e.getY() );
                if ( selected != null ){
                    ConceptOptions opciones = new ConceptOptions ( selected , MapWindow.this.mapCanvas );
                    opciones.setVisible( true );
                }
               
            }
            else if ( (MapWindow.this.mapCanvas.getMouseMode() == MapWindow.this.mapCanvas.SELECT_RELATIONS || MapWindow.this.mapCanvas.getMouseMode() == MapWindow.this.mapCanvas.CREATE_RELATIONS) && ( e.getButton() == e.BUTTON2 || e.getButton() == e.BUTTON3 )  ){
                Map.Relation selected = MapWindow.this.mapCanvas.getMap().findRelation( e.getX() , e.getY() );
                if ( selected != null ){
                    RelationOptions opciones = new RelationOptions ( selected , MapWindow.this.mapCanvas );
                    opciones.setVisible( true );
                } 
            }
            
        } 
        
        /**
         * evento se salir del lienzo
         * @param e evento del raton
         */
        public void mouseExited( java.awt.event.MouseEvent e) {
            if ( MapWindow.this.conceptToDrag != null )
                MapWindow.this.conceptToDrag = null;
            
            if ( MapWindow.this.mapCanvas.creatingRelation ){
                MapWindow.this.mapCanvas.startingConcept = null;
                MapWindow.this.mapCanvas.creatingRelation = false;
                MapWindow.this.mapCanvas.repaint();
            }
        }
    }
    
    /**
     * Escucha para los cambios en el menu Archivo
     */
    private class EscuchaMenuAbrir implements java.awt.event.ActionListener {
        /**
         * Accion de hacer click sobre cualquier elemento en el menu archivo
         * @param ae evento
         */
        public void actionPerformed ( java.awt.event.ActionEvent ae ){
            String arg = (String) ae.getActionCommand();
            if ( arg.equals("Salir") ){
                MapWindow.this.dispose();
            }
            else if ( arg.equals("Abrir") ){
                java.awt.Frame f = new java.awt.Frame();
                java.awt.FileDialog dialogo = new java.awt.FileDialog(f , "Abrir Mapa" , java.awt.FileDialog.LOAD);
                dialogo.setVisible( true );
                MapWindow.this.mapCanvas.getMap().load (dialogo.getDirectory(),dialogo.getFile());
                MapWindow.this.mapCanvas.repaint();
                MapWindow.this.iteracion.setText( MapWindow.this.mapCanvas.getMap().getIterationLabel());
                if ( dialogo.getFile() != null )
                    if ( MapWindow.this.mapCanvas.getMap().isDynamic() )
                        MapWindow.this.setTitle("DFCM Designer - " + dialogo.getFile() + " - Versin 1.0");
                    else
                        MapWindow.this.setTitle("FCM Designer - " + dialogo.getFile() + " - Versin 1.0");
            }
            else if ( arg.equals("Guardar") ){
                java.awt.Frame f = new java.awt.Frame();
                java.awt.FileDialog dialogo = new java.awt.FileDialog(f , "Guardar Mapa" , java.awt.FileDialog.SAVE );
                dialogo.setVisible( true );
                MapWindow.this.mapCanvas.getMap().save (dialogo.getDirectory(),dialogo.getFile());
                if ( dialogo.getFile() != null )
                    if ( MapWindow.this.mapCanvas.getMap().isDynamic() )
                        MapWindow.this.setTitle("DFCM Designer - " + dialogo.getFile()+ " - Versin 1.0" );
                    else
                        MapWindow.this.setTitle("FCM Designer - " + dialogo.getFile() + " - Versin 1.0");
            }
            else if ( arg.equals ("Nuevo") ){
                MapWindow.this.mapCanvas.getMap().clear();
                MapWindow.this.mapCanvas.repaint();
                MapWindow.this.iteracion.setText( MapWindow.this.mapCanvas.getMap().getIterationLabel());
            }
            else if ( arg.equals ("Guardar Ejecucin") ){
                java.awt.Frame f = new java.awt.Frame();
                java.awt.FileDialog dialogo = new java.awt.FileDialog(f , "Guardar Ejecucion" , java.awt.FileDialog.SAVE );
                dialogo.setVisible( true );
                MapWindow.this.mapCanvas.getMap().saveExecution (dialogo.getDirectory(),dialogo.getFile());
            } 
        }
    }
    
    /**
     * Escucha para indicar que tipo de visualizacion de los conceptos ( proporcional o fijo )
     */
    private class EscuchaMenuVista implements java.awt.event.ItemListener{
        /**
         * Opcion 1
         */
        private java.awt.CheckboxMenuItem op1;
        /**
         * Opcion 2
         */
        private java.awt.CheckboxMenuItem op2;
        /**
         * Constructor indica una referencia a las distintas opciones
         * @param op1 Opcion 1
         * @param op2 Opcin 2
         */
        public EscuchaMenuVista ( java.awt.CheckboxMenuItem op1 , java.awt.CheckboxMenuItem op2 ){
            this.op1 = op1;
            this.op2 = op2;
        }
        
        /**
         * Evento de cambiar de estado
         * @param evt evento
         */
        public void itemStateChanged( java.awt.event.ItemEvent evt ) {
            if ( evt.getItem().equals( op1.getLabel() ) ){
                if ( evt.getStateChange() == evt.DESELECTED ){
                    op2.setState( true );
                    MapWindow.this.mapCanvas.dibujarPorpocionalmente = false;
                }
                else {
                    op2.setState( false );
                    MapWindow.this.mapCanvas.dibujarPorpocionalmente = true;
                }
            }
            else if ( evt.getItem().equals( op2.getLabel() ) ){
                if ( evt.getStateChange() == evt.DESELECTED ){
                    op1.setState( true );
                    MapWindow.this.mapCanvas.dibujarPorpocionalmente = true;
                }
                else {
                    op1.setState( false );
                    MapWindow.this.mapCanvas.dibujarPorpocionalmente = false;
                }
            }
            
            MapWindow.this.mapCanvas.repaint();
        }
    }
    
    /**
     * Escucha para el boton de ejecutar el mapa
     */
    private class EscuchaBotonPLay extends java.awt.event.MouseAdapter{ 
        java.awt.TextField entradaIteracion;
        public EscuchaBotonPLay (java.awt.TextField entradaIteracion){
            this.entradaIteracion = entradaIteracion;
        }
        
        /**
         * evento de cliquear el raton
         * @param e evento
         */
        public void mouseClicked( java.awt.event.MouseEvent e) {
            if ( ((java.awt.Button)e.getSource()).getLabel().equals("Reset") ){
                ((java.awt.Button)e.getSource()).setLabel( "Play" );
                MapWindow.this.mapCanvas.getMap().resetConcepts();
                MapWindow.this.mapCanvas.repaint();
                MapWindow.this.iteracion.setText( MapWindow.this.mapCanvas.getMap().getIterationLabel() );
                this.entradaIteracion.setText( "0" );
            }
            else if ( ((java.awt.Button)e.getSource()).getLabel().equals("Play") ){
                MapWindow.this.hilo.run();
            }
        } 
    }
    
    /**
     * Escucha para la visualizacin de las iteraciones
     */
    private class EscuchaVistaIteracion extends java.awt.event.MouseAdapter { 
        private java.awt.TextField entradaIteracion;
        
        public EscuchaVistaIteracion ( java.awt.TextField entradaIteracion ){
            this.entradaIteracion = entradaIteracion;
        }
        
        public void mouseClicked( java.awt.event.MouseEvent e){
            if ( ( (java.awt.Button)e.getSource() ).getLabel().equals(">>") )
                MapWindow.this.mapCanvas.getMap().setShowIteration( MapWindow.this.mapCanvas.getMap().getCurrentIteration()+1);
            
            else if ( ( (java.awt.Button)e.getSource() ).getLabel().equals("<<") )
                MapWindow.this.mapCanvas.getMap().setShowIteration( MapWindow.this.mapCanvas.getMap().getCurrentIteration()-1);
            
            else if ( ( (java.awt.Button)e.getSource() ).getLabel().equals("Ver") )
                MapWindow.this.mapCanvas.getMap().setShowIteration(Integer.valueOf(this.entradaIteracion.getText()).intValue());
            
            MapWindow.this.mapCanvas.repaint();
            MapWindow.this.iteracion.setText( MapWindow.this.mapCanvas.getMap().getIterationLabel() );
        }     
    }
    
    /**
     * Escucha para las opciones de ejecucin
     */
    private class EscuchaMenuEjecucion implements java.awt.event.ItemListener{
        java.awt.CheckboxMenuItem op1;
        java.awt.CheckboxMenuItem op2;
        java.awt.CheckboxMenuItem op3;
        java.awt.CheckboxMenuItem op4;
        public EscuchaMenuEjecucion ( java.awt.CheckboxMenuItem op1 , java.awt.CheckboxMenuItem op2, 
                       java.awt.CheckboxMenuItem op3 , java.awt.CheckboxMenuItem op4 ){
            this.op1 = op1;
            this.op2 = op2;
            this.op3 = op3;
            this.op4 = op4;
        }
        
        public void itemStateChanged( java.awt.event.ItemEvent evt ) {
            if ( evt.getItem().equals( this.op1.getLabel()) ){
                op1.setState(true);
                op2.setState(false);
                op3.setState(false);
                op4.setState(false);
                MapWindow.this.hilo.tipoSaturacion = MapWindow.this.hilo.BISTATE;
            }
            else if ( evt.getItem().equals( this.op2.getLabel()) ){
                op1.setState(false);
                op2.setState(true);
                op3.setState(false);
                op4.setState(false);
                MapWindow.this.hilo.tipoSaturacion = MapWindow.this.hilo.TRISTATE;
            }
            else if ( evt.getItem().equals( this.op3.getLabel()) ){
                op1.setState(false);
                op2.setState(false);
                op3.setState(true);
                op4.setState(false);
                MapWindow.this.hilo.tipoSaturacion = MapWindow.this.hilo.SYC;
            }
            else if ( evt.getItem().equals( this.op4.getLabel()) ){
                op1.setState(false);
                op2.setState(false);
                op3.setState(false);
                op4.setState(true);
                MapWindow.this.hilo.tipoSaturacion = MapWindow.this.hilo.SY;
            }
        }
    }
}
