package Map;

/**
 * Objeto Utilizado para Represenatr las relaciones causales entre
 * Dos Conceptos Dados
 * Created on 30 de enero de 2005, 23:01
 * @author Jos Bladimir Contreras Vargas
 * @author <A HREF="mailto:jbc_ve@hotmail.com">jbc_ve@hotmail.com</A>
 */
public class Relation {
    /**
     * Identificador de las Relaciones
     */
    static private int relationID = 0;
    /**
     * identificador de la relacion
     */
    private int identificador;
    /**
     * valor de la relacin
     */
    private float valor = 0.0f;
    /**
     * punto de dibujo de la relacin
     */
    private java.awt.Point puntoDibujo;
    /**
     * Concepto Incial
     */
    private Concept initialConcept;
    /**
     * Concepto Final
     */
    private Concept finalConcept;
    /**
     * indica si el concepto pertenece a un mapa dinamico o no
     */
    private boolean dynamic = false;
    /**
     * indica que porcentaje de aproximacin de la etiqueta al nodo concecuente
     */
    private float porcentajeAproximacion = 0.8f;
    /**
     * indica el dinamicImput de la relacin
     */
    private float dinamicImput = 1.0f;
    /**
     * Comentario de la relacin
     */
    public String comentario = "";
    
    /**
     * Crea una nueva relacin
     * @param nodoInicial Concepto Antecedente
     * @param nodoFinal Concepto Concecuente
     * @param dynamic Indica si la relacin pertenece a un mapa dinamico o no
     */
    public Relation(Concept nodoInicial , Concept nodoFinal, boolean dynamic) { 
        this.initialConcept = nodoInicial;
        this.finalConcept = nodoFinal;
        this.valor = 1.0f;
        this.identificador=this.relationID++;
        this.dynamic = dynamic;
    }
    
    /**
     * Asigna un dinamicImput a la relacin
     * @param p dinamicImput a asignar
     */
    public void setDinamicImput ( float p ){
        this.dinamicImput = p;
    }
    
    /**
     * Retorna el dinamicImput de la relacin
     * @return dinamicImput de la relacin
     */
    public float getDinamicImput (){
        return this.dinamicImput;
    }
    
    /**
     * retorna si la relacin pertenece a una mapa dinamico o statico
     * @return True - indica dinamic
     * false - indica statica
     */
    public boolean isDynamic (){
        return this.dynamic;
    }
    
    
    
    /**
     * Retorna el concepto Antecedente de la relacin
     * @return concepto antecedente de la relacion
     */
    public Concept getInitialConcept (){
        return this.initialConcept;
    }
    
    /**
     * Retorna el concepto consecuente de la relacion
     * @return concepto consecuente de la relacion
     */
    public Concept getFinalConcept (){
        return this.finalConcept;
    }
    
    /**
     * retorna el valor de la relacin
     * @return valor de la relacin
     */
    public float getValue (){
        return this.valor;
    }
    
    /**
     * Asigna el valor de la relacion
     * @param f valor de la relacin debe estar entre los valores [0 1]
     */
    public void setValor ( float f ){
        if ( !(f < -1) && !( f > 1 ) )
            this.valor = f;
    }
    
    /**
     * obtiene el porcentaje de dibujo de la relacion
     * @return porcentaje de aproximacin para el dibujo
     */
    public float getPorcentajeAproximacion (){
        return this.porcentajeAproximacion;
    }
    
    /**
     * asigan el porcentaje de dibujo de la relacin
     * @param p porcentaje de dibujo de la relacin
     */
    public void setPorcentajeAproximacion ( float p ){
        if ( !(p < 0) && !( p > 1 ) )
            this.porcentajeAproximacion = p;
    }
       
    /**
     * obtiene el identificador de la relacin
     * @return identificador de la relacin
     */
    public int getRelationId (){
        return this.identificador;
    }
    
    /**
     * Dibuja la relacion
     * @param g Objeto grafico donde se debe dibujar la relacin
     */
    public void drawRelation ( java.awt.Graphics g  ){
  
        double angle = Math.atan2( initialConcept.getPosition().y - finalConcept.getPosition().y ,
                                   initialConcept.getPosition().x - finalConcept.getPosition().x );
        
        java.awt.Point pi = new java.awt.Point ( (int) Math.round( initialConcept.getPosition().x - initialConcept.getRadio() * Math.cos(angle) + initialConcept.getRadio() ) ,
                               (int) Math.round( initialConcept.getPosition().y - initialConcept.getRadio() * Math.sin(angle) + initialConcept.getRadio() )  );
        
        java.awt.Point pf = new java.awt.Point ( (int) Math.round( finalConcept.getPosition().x + finalConcept.getRadio() * Math.cos(angle) + finalConcept.getRadio() ), 
                               (int) Math.round( finalConcept.getPosition().y + finalConcept.getRadio() * Math.sin(angle) + finalConcept.getRadio() ) );
        
        g.drawLine( pi.x , pi.y , pf.x , pf.y );

        float x1 = pi.x, y1 = pi.y , x2 = pf.x , y2 = pf.y;
        
        float xd = Math.abs(x1-x2) * porcentajeAproximacion ;
        
        if ( x1 < x2 )
            xd += x1;
        else
            xd = x1-xd;
        
        float yd ;
        
        if ( x1 == x2 ){
            yd = Math.abs( y1 - y2 ) * this.porcentajeAproximacion;
            
            if ( y1 < y2 )
                yd += y1;
            else
                yd = y1-yd;
        }
        else
            yd = ((y2-y1)/(x2-x1)) * xd + (x2*y1-x1*y2)/(x2-x1);
        
        this.puntoDibujo = new java.awt.Point ( Math.round( xd ), Math.round( yd ) ); 
        
        String v = Float.toString( this.valor );
        
        if ( this.dynamic )
            v = Float.toString( Algorithms.Rules.computeRelationValue( this ) );

        while ( v.length() <= 5 )
            v += '0';
        
        if ( v.length() > 5 )
            v = v.substring( 0 , 5 );
        
        if ( this.valor < 0.0f )
            g.setColor(java.awt.Color.RED);
        else if ( this.valor > 0.0f )
            g.setColor(java.awt.Color.BLUE);
        else
            g.setColor(java.awt.Color.LIGHT_GRAY);
        
        /******DIBUJO de la flecha********/
        float b = (float)(Math.cos(Math.PI/9.0)*15.0);
        float a = (float)(Math.sin(Math.PI/9.0)*15.0);
        float zx = (float)(pf.x + Math.cos(angle) * b);
        float zy = (float)(pf.y + Math.sin(angle)*b);
        float dx = (float) Math.cos(Math.PI/2.0+angle)*a;
        float dy = (float)Math.sin(Math.PI/2.0+angle)*a;
        
        float P1x = zx - dx;
        float P1y = zy - dy;
        float P2x = zx + dx;
        float P2y = zy + dy;
        
        g.drawLine( Math.round(P1x) , Math.round(P1y) , pf.x , pf.y );
        g.drawLine( Math.round(P2x) , Math.round(P2y) , pf.x , pf.y );
        g.fillOval( pf.x-3 , pf.y-3 , 6 , 6 );
        /******end DIBUJO de la flecha********/

        g.drawString( v , this.puntoDibujo.x+5 , this.puntoDibujo.y );

        g.setColor( java.awt.Color.BLACK );        
    }
    
    /**
     * indica si la posicion se encuentra dentro del rectangulo de la relacin
     * @param x posicin x
     * @param y posicin y
     * @return true - si la posicin se encuentra dentro del rectangulo que engloba la relacion
     * false - si la posicin se encuentra fuera del rectangulo que engloba la relacion
     */
    public boolean dentro ( int x , int y ){
        return ((x >= this.puntoDibujo.x+5    && x <= this.puntoDibujo.x+45) && 
                (y >= this.puntoDibujo.y - 10 && y <= this.puntoDibujo.y ) );
    }
    
    /**
     * Asigna un comentario a la relacion
     * @param c String con el comentario que identifica a la relacion no se puede usar el caracter &
     */
    public void setComentario ( String c ){
        this.comentario = c.replace( '\n','&');
        this.comentario = this.comentario.replace((char)13 , ' ');
    }
    
    /**
     * retorna el comentario de la relacion
     * @return String que contiene el comentario de la relacion
     */
    public String getComentario (){
        return this.comentario.replace( '&' , '\n');
    }
}
